/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2005 Cavium Networks
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#define __USE_GNU
#include <sched.h>

int main(int argc, char* argv[])
{
    unsigned long cpumask;

    if (argc < 3)
    {
        printf("\n"
               "Usage: oncpu <cpu> <command>\n"
               "       oncpu <mask> <command>\n"
               "\n"
               "Starts the supplied command on a specific cpu or limits the\n"
               "command to running on the cpus in the mask. 'cpu' must be a\n"
               "decimal integer from 0..CPUS-1. 'mask' must be a hexadecimal\n"
               "number specified as 0xXXXX.\n"
               "\n");
        return -1;
    }

    if ((argv[1][0] == '0') && (argv[1][1] == 'x'))
        sscanf(argv[1]+2, "%lx",&cpumask);
    else
        cpumask = 1 << atoi(argv[1]);

    printf("Setting affinity to 0x%lx\n", cpumask);
    if (sched_setaffinity(0, sizeof(cpumask), (cpu_set_t*)&cpumask))
    {
        perror("Error setting affinity");
        return -1;
    }

    if (execvp(argv[2], argv+2))
        perror("Exec faild");

    return -1;
}
