/*
 *  ebt_rem1q
 *
 *  Copyright (c) Arada Systems, 1996-2007
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/ether.h>
#include <getopt.h>
#include <ctype.h>
#include "ebtables_u.h"
#include <linux/netfilter_bridge/ebt_8021q.h>

#define REM1Q_TARGET 		'1'
#define OPT_REM1Q_TARGET	0x01

static struct option opts_rem1q[] =
{
	{ "rem1q-target", required_argument, 0, REM1Q_TARGET },
	{ 0 }
};

static void print_help_rem1q()
{
	printf(
	"rem1q option:\n"
	" --rem1q-target target   : ACCEPT, DROP, RETURN or CONTINUE\n"
	"default target = CONTINUE\n"
	"This strips a 802.1q tag\n");
}

static void init_rem1q(struct ebt_entry_target *target)
{
	struct ebt_8021q_info *rem1q_info = (struct ebt_8021q_info *)target->data;

	rem1q_info->target = EBT_CONTINUE;
	return;
}

static int parse_rem1q(int c, char **argv, int argc, const struct ebt_u_entry *entry, unsigned int *flags, struct ebt_entry_target **target)
{
	struct ebt_8021q_info *rem1q_info =
	   (struct ebt_8021q_info *)(*target)->data;


	switch (c) {
	case REM1Q_TARGET:
		ebt_check_option2(flags, OPT_REM1Q_TARGET);
		if (FILL_TARGET(optarg, rem1q_info->target))
			ebt_print_error("Illegal --rem1q-target target");
		break;
	default:
		return 0;
	}
	return 1;
}

static void final_check_rem1q(const struct ebt_u_entry *entry,
   const struct ebt_entry_target *target, const char *name,
   unsigned int hookmask, unsigned int time)
{
	struct ebt_8021q_info *rem1q_info =
	   (struct ebt_8021q_info *)target->data;

	if (BASE_CHAIN && rem1q_info->target == EBT_RETURN)
		ebt_print_error("--rem1q-target RETURN not allowed on "
		            "base chain");
	CLEAR_BASE_CHAIN_BIT;
	if ( ((hookmask & ~(1 << NF_BR_PRE_ROUTING)) || strcmp(name, "nat")) &&
	   ((hookmask & ~(1 << NF_BR_BROUTING)) || strcmp(name, "broute")) )
		ebt_print_error("Wrong chain for remove-vlantag");
}

static void print_rem1q(const struct ebt_u_entry *entry, const struct ebt_entry_target *target)
{
	struct ebt_8021q_info *rem1q_info =
	   (struct ebt_8021q_info *)target->data;

	if (rem1q_info->target == EBT_ACCEPT)
		return;
	printf(" --rem1q-target %s", TARGET_NAME(rem1q_info->target));
}

static int compare_rem1q(const struct ebt_entry_target *t1, const struct ebt_entry_target *t2)
{
	struct ebt_8021q_info *rem1q_info1 =
	   (struct ebt_8021q_info *)t1->data;
	struct ebt_8021q_info *rem1q_info2 =
	   (struct ebt_8021q_info *)t2->data;

	return rem1q_info1->target == rem1q_info2->target;
}

static struct ebt_u_target rem1q_target =
{
	.name		= EBT_REM1Q_TARGET,
	.size 		= sizeof(struct ebt_8021q_info),
	.help 		= print_help_rem1q,
	.init 		= init_rem1q,
	.parse 		= parse_rem1q,
	.final_check 	= final_check_rem1q,
	.print 		= print_rem1q,
	.compare 	= compare_rem1q,
	.extra_ops 	= opts_rem1q,
};

void _init(void)
{
	ebt_register_target(&rem1q_target);
}
