#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <net/if.h>
#include "br_private.h"
#include <linux/if_bridge.h>
#include <linux/sockios.h>
#include "bridge_struct.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#define MAX_CMD_LEN 128
#define MAX_PORTS 256
#define MAX_NAME_SZ 128
//#define DEBUG_MSG (fmt,args...) snmp_log(LOG_ERR,fmt,args)
#if 0
#define DEBUG_MSG(fmt,args...) printf(fmt,args)
#else
#define DEBUG_MSG(fmt,args...)
#endif

int open_socket()
{
	int skfd = -1;
	if((skfd = socket(AF_INET, SOCK_DGRAM,0) ) < 0)
		return -1;
	else
		return skfd;	
}

int read_brname(char *brname)
{
        FILE *fp;
        char cmd[MAX_CMD_LEN] = "/usr/local/sbin/brctl show | awk '{if(NR == 2) print $1}'";
        int i;

	if( (fp = popen(cmd,"r")) < 0)
        {
                return -1;
        }

	if( !(fgets(brname,128,fp)) )
        {
                fclose (fp);
                return -1;
        }	

	for(i=0; i < strlen(brname) ; i++)
        {
                if(brname[i] == '\n')
                {
                        brname[i] = '\0';
                        break;
                }
        }

        fclose (fp);
	return 0;
}


int brdg_info(bridge_info *br_info)
{
	int skfd = -1;
        char brname[MAX_NAME_SZ],*buf;
	struct ifreq ifr;
        struct __bridge_info br;
        int ifindices[MAX_PORTS];
        int num_ports = 0;
        unsigned long args[4] = {BRCTL_GET_BRIDGE_INFO,(unsigned long) &br,0,0};
        int i;
	
	skfd = open_socket();
	if(skfd < 0)
		return -1;
	if(read_brname(brname) < 0)
	{
		close(skfd);
		return -1;
	}

	DEBUG_MSG("\nBridge Name %s",brname);
		
	strcpy(ifr.ifr_name,brname);
        ifr.ifr_data = (char *)args;
	/* Addr, Id and Priority*/
	if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
        {
                DEBUG_MSG("Ioctl failed\n",1);
		close(skfd);
                return -1;
        }

	DEBUG_MSG("brdige_id 0x%llx\n",(br.bridge_id));
	buf = (char *)&(br.bridge_id);
	for(i = 0;i < 8; i++)
	{
		br_info->brdg_id[i] = buf[i]; 
	}
	
	DEBUG_MSG("bridge_addr = ",1);
        for (i = 0;i < 6; i++)
	{
        	DEBUG_MSG(":0x%x",( (bridge_id *)&(br.bridge_id))->addr[i]);
		br_info->macaddr[i] = ((bridge_id *)&(br.bridge_id))->addr[i];
	}

	DEBUG_MSG("brdige_priority 0x%x\n",htons(*(short int *)( (bridge_id *)&br.bridge_id)->prio) );
	br_info->priority = htons(*(short int *)( (bridge_id *)&br.bridge_id)->prio);
	
 
	DEBUG_MSG("\nDesignated root = 0x%llx\n",br.designated_root);
	buf = (char *)&(br.designated_root);
        for(i = 0;i < 8; i++)
        {
		br_info->designated_root[i] = buf[i];
	}

	DEBUG_MSG("Time Since Topology change TBD\n",1);
	br_info->time_since_tp_ch = br.topology_change_timer_value;	/*TBD*/
	
	DEBUG_MSG("Topology change counter\n",1);
	br_info->tp_count = 0;					/*TBD*/

        DEBUG_MSG("root_path_cost = 0x%x\n",(br.root_path_cost));
	br_info->root_cost = br.root_path_cost;

	DEBUG_MSG("root_port = %d\n",br.root_port);
	br_info->root_port = br.root_port;
	
	DEBUG_MSG("br.max_age = %u\n",br.max_age);
	br_info->max_age = br.max_age;
	
	DEBUG_MSG("br.hello_time = %u\n",br.hello_time);
	br_info->hello_time = br.hello_time;

	DEBUG_MSG("Hold time TBD\n",1);
	br_info->hold_time = 0;			/*TBD*/
	
	DEBUG_MSG("br.forward_delay = %u\n",br.forward_delay);
	br_info->fwrd_delay = br.forward_delay;

	DEBUG_MSG("br.bridge_max_age = %u\n",br.bridge_max_age);
	br_info->brmax_age = br.bridge_max_age;

	DEBUG_MSG("br.bridge_hello_time %u\n",br.bridge_hello_time);
	br_info->brhello_time = br.bridge_hello_time;

	DEBUG_MSG("br.bridge_forward_delay = %u\n",br.bridge_forward_delay);
	br_info->brfwrd_delay = br.bridge_forward_delay;

	br_info -> entry_discard = 0;			/*TBD*/		
	br_info -> ageing_time = br.ageing_time;	/*TBD*/	

	DEBUG_MSG("Base type\n",1);
	br_info -> base_type = 2;		/*Transparent-only*/	

	DEBUG_MSG("protcol specification\n",1);
	br_info->prot_spec = 3;			/*ieee8021d*/

	args[0] = BRCTL_GET_PORT_LIST;
        memset(ifindices,0,(MAX_PORTS * 4));
        args[1] = (unsigned long) ifindices;

        strcpy(ifr.ifr_name,brname);
        ifr.ifr_data = (char *)args;
        if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
	{
		close(skfd);
                return -1;
	}
        num_ports = 1;
        while(ifindices[num_ports])
        {
        	num_ports++;
        }
        num_ports--;
        DEBUG_MSG("\nnumports = %d\n",num_ports);
	br_info -> numports = num_ports;
	close(skfd);
}


int base_port_table (base_port **head)
{
	
	int skfd = -1;
        char brname[MAX_NAME_SZ];
	struct ifreq ifr;
        int ifindices[MAX_PORTS];
        int num_ports;
        unsigned long args[4] = {BRCTL_GET_PORT_LIST,(unsigned long) ifindices,0,0};
	base_port *tmp,*node;
        int i;
	
	DEBUG_MSG("\nBase Port Bridge Name %s",brname);

	
	skfd = open_socket();
	if(skfd < 0)
		return -1;
	if(read_brname(brname) < 0)
	{
		close(skfd);
		return -1;
	}

	if((*head))
	{
	DEBUG_MSG("\nBase Port head free it\n",1);
		for(node=(*head) ; node ; )
		{
			tmp = node;
			node = node -> next;
			free(tmp);
		}
	*head = NULL; 
	}
	
		
	strcpy(ifr.ifr_name,brname);
        ifr.ifr_data = (char *)args;
        if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
	{
		close(skfd);
                return -1;
	}
        num_ports = 1;
        while(ifindices[num_ports])
        {
        	num_ports++;
        }
        num_ports--;

        DEBUG_MSG("\nnumports = %d\n",num_ports);
	tmp = node = NULL;
	if(num_ports)
        {
                for(i=1;i <= num_ports;i++)
                {
			node = (base_port *)malloc(sizeof(base_port));
			if(!node)
			{
				snmp_log(LOG_ERR,"\nmalloc for node failed\n");
				break;
			}
			if(i == 1)
				*head = node;
			if(tmp)
				tmp->next = node;
			tmp = node;
			node->next = NULL;
                        DEBUG_MSG("port_idx:%d ifindex:%d\t",i,ifindices[i]);
			node -> port_idx = i;
			node -> ifindex = ifindices[i];
			memset(node -> circuit,0,4);
			node -> delay_excd_discard = 0;		/*TBD*/
			node -> mtu_excd_discard = 0;		/*TBD*/
		}
	}
	close(skfd);
	return 0;
}

int stp_port_table (stp_port **head)
{
	int skfd = -1;
        char brname[MAX_NAME_SZ],*buf;
	int num_ports;
        struct __port_info prt;
        struct ifreq ifr;
        int ifindices[MAX_PORTS];
        unsigned long args[4] = {BRCTL_GET_PORT_LIST,(unsigned long) ifindices,0,0};
        int i;
	stp_port *node,*tmp;
	
	skfd = open_socket();
        if(skfd < 0)
                return -1;
        if(read_brname(brname) < 0)
        {
                close(skfd);
                return -1;
        }
	
        memset(ifindices,0,(MAX_PORTS * 4));
        strcpy(ifr.ifr_name,brname);
        ifr.ifr_data = (char *)args;
        if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
	{
		close(skfd);
                return -1;
	}

	if(*head)
        {
                for(node=*head ; node ; )
                {
                        tmp = node;
                        node = node -> next;
                        free(tmp);
                }
	*head = NULL;
        }
	
        num_ports = 1;
        while(ifindices[num_ports])
                {
                        num_ports++;
                }
        num_ports--;
        DEBUG_MSG("%d\n",num_ports);
	tmp = NULL;
	if(num_ports)
        {
                strcpy(ifr.ifr_name,brname);
                args[0] = BRCTL_GET_PORT_INFO;
                args[1] = (unsigned long) &prt;
                for(i=1;i <= num_ports;i++)
                {
                        DEBUG_MSG("\n-----------------------------\n",1);
                        DEBUG_MSG("port_idx:%d ifindex:%d\n",i,ifindices[i]);
                        args[2] = i;		/*Port Num to be queried*/
                        ifr.ifr_data = (char *)args;
                        if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
			{
				close(skfd);
                                return -1;
			}
			
			node = (stp_port *)malloc(sizeof(stp_port));
                        if(!node)
                                break;
                        if(i == 1)
                                *head = node;
                        if(tmp)		/*tmp = previous node*/
                                tmp->next = node;
                        tmp = node;
                        node->next = NULL;
                        
			node -> port_idx = i;
			node -> priority = 0;	/*TBD*/
			node -> enable  = 0;	/*TBD*/
			DEBUG_MSG("state = %d",prt.state);
			node -> state = prt.state + 1;
                        DEBUG_MSG("path_cost = %u\n",prt.path_cost);
			node -> path_cost = prt.path_cost;
                        DEBUG_MSG("designated_root = 0x%llx\n",prt.designated_root);
			buf = (char *)&(prt.designated_root);
       			for(i = 0;i < 8; i++)
        		{
				node->designated_root[i] = buf[i];
			}
		
                        DEBUG_MSG("designated_cost = %u\n",prt.designated_cost);
			node -> designated_cost = prt.designated_cost;
                        DEBUG_MSG("designated_bridge = 0x%llx\n",prt.designated_bridge);
       			for(i = 0;i < 8; i++)
        		{
				node->bridge_id[i] = buf[i];
			} 
                        DEBUG_MSG("designated_port = %d\n",prt.designated_port);
			buf =  (u_char *)&prt.designated_port;
			node -> designated_port[0] = buf[1];
			node -> designated_port[1] = buf[0];
			
			node -> frwd_transition = 0;	/*TBD*/
                }

        }
}

int dtpfdp_table(dtp_fdb **head)
{
	int skfd = -1;
        char brname[MAX_NAME_SZ],*buf;
        int num_entries = 0;
        struct ifreq ifr;
        struct __fdb_entry fdb;
        unsigned long args[4] = {BRCTL_GET_FDB_ENTRIES,(unsigned long) &fdb,0,0};
        FILE *fp;
        char cmd[MAX_CMD_LEN] = "/usr/local/sbin/brctl showmacs ";
        char line[128];
        int i;
	dtp_fdb *node,*tmp;	
	skfd = open_socket();
        if(skfd < 0)
                return -1;
        if(read_brname(brname) < 0)
        {
                close(skfd);
                return -1;
        }


        strcat (cmd,brname);
        /*printf("\ncmd =%s\n",cmd);*/
        fp = popen(cmd,"r");
        if(fp == NULL)
        {
                snmp_log(LOG_ERR,"popen failed\n");
                return -1;
        }

	if(*head)
        {
                for(node=*head ; node ; )
                {
                        tmp = node;
                        node = node -> next;
                        free(tmp);
                }
        *head = NULL;
        }

        while(fgets(line,128,fp))
        {
                num_entries++;
        }
        num_entries--;
	DEBUG_MSG("\nNum of entries = %d\n",num_entries);

	tmp = NULL;
        strcpy(ifr.ifr_name,brname);
        for (i = 0; i < num_entries; i++)
        {
                args[2] = 1;	/*Num of Entries to be read*/
                args[3] = i;	/*offset*/
                ifr.ifr_data = (char *)args;
                if(ioctl(skfd,SIOCDEVPRIVATE,&ifr) < 0)
		{
			snmp_log(LOG_ERR,"Ioctl failed FDB\n");
			fclose(fp);
			close(skfd);
                        return -1;
		}

		node = (dtp_fdb *)malloc(sizeof(dtp_fdb));
                if(!node)
		{
			snmp_log(LOG_ERR,"\nAlloc failed for fdb entry\n");
                	break;
		}
                if(i == 0)
                        *head = node;
                if(tmp)         /*tmp = previous node*/
                       tmp->next = node;
                tmp = node;
                node->next = NULL;

                DEBUG_MSG("\n##################################\n",1);
		for(i = 0; i < 6 ; i++)
			node -> macaddr[i] = fdb.mac_addr[i];
                DEBUG_MSG("mac_addr %2x:",fdb.mac_addr[0]);
                DEBUG_MSG("%2x:",fdb.mac_addr[1]);
                DEBUG_MSG("%2x:",fdb.mac_addr[2]);
                DEBUG_MSG("%2x:",fdb.mac_addr[3]);
                DEBUG_MSG("%2x:",fdb.mac_addr[4]);
                DEBUG_MSG("%2x\n",fdb.mac_addr[5]);
	
                DEBUG_MSG("port_no %d\n",fdb.port_no);
		node -> port = fdb.port_no;

                DEBUG_MSG("is_local %d\n",fdb.is_local);
                DEBUG_MSG("ageing_timer_value %x\n",fdb.ageing_timer_value);
		node -> status = (fdb.is_local & fdb.ageing_timer_value);
        }

	close(skfd);
	fclose(fp);
	return 0;
}
