#include <string.h>
#include <stdlib.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#include "ieee80211_ioctl.h"
#include "ieee802dot11_info.h"
#include "iwlib.h"
#include "AuthenticationAlgo.h"
#include "DataRates.h"
#include "WEPDefaultKey.h"
#include "StationConfigTable.h"
#include "Privacy.h"
#include "PhyOperation.h"
#include "MultiDoamianCapability.h"
#include "PhyHT.h"
#include "HTStationConfig.h"
#include "OpTable.h"
#include "QAPEDCATable.h"
#include "QCTable.h"
#include "CntTable.h"
#include "MCSTxTable.h"
#include "MCSRxTable.h"
#include "PhyTxPower.h"

#define ERR -1
#define DEBUG_MSG(fmt,args...) printf(fmt,##args)

void flushlist(list_T *list_ptr)
{
	struct list_node *node = NULL,*tmp = NULL;
	if(!(list_ptr->l_head))
		return;
	node = list_ptr->l_head;
	while(node)
	{
		tmp = node;
		if(tmp->data)
			free(tmp->data);
		node = tmp->next;
		free(tmp);
	}
	return;
}

void list_int (list_T *list_ptr)
{
	LIST_INIT (list_ptr);
}

void addList (list_T *List,char *data,int len)
{
	struct list_node *newNode,*nodeP;
        newNode = (struct list_node *) malloc(sizeof(struct list_node));
	if(!newNode) {
		DEBUG_MSG("\naddList -newNode: malloc Failed");	
		return;
	}
        newNode->data = (char *) malloc(len);
	if(!newNode->data){
		DEBUG_MSG("\naddList :malloc Failed");	
		return;
	}
        memcpy ( newNode->data, data, len );
        newNode->next = NULL;
        if(LIST_EMPTY(List)) 
	{
                List->l_head = newNode;
        }
	else 
	{
                nodeP = List->l_head;
                while(nodeP->next != NULL)
                        nodeP = nodeP->next;
                nodeP->next = newNode;
        }
	
}

cntTbl_data* cntList_get(list_T *cntList,int index,int exact)
{
        struct list_node *np;
        cntTbl_data* cntTbl_P;
        int flag = 0;
        for ( np = LIST_FIRST ( cntList ); np != NULL; np = np->next ) {
                cntTbl_P = (cntTbl_data*) np->data;
                if(flag)
                        return cntTbl_P;
                if(cntTbl_P->ifIndex == index) {
                        if(exact)
                                return cntTbl_P;
                        else
                                flag = 1;
                }
        }
        return NULL;
}



qcTbl_data* qcList_get(list_T *qcList,int ifIndex,int qcIndex,int exact)
{
        struct list_node *np;
        qcTbl_data* qcTbl_P;
        int flag = 0;
        for ( np = LIST_FIRST ( qcList ); np != NULL; np = np->next ) {
                qcTbl_P = (qcTbl_data*) np->data;
                if(flag)
                        return qcTbl_P;
                if((qcTbl_P->ifIndex == ifIndex)&&(qcTbl_P->QosCountersIndex == qcIndex)) {
                        if(exact)
                                return qcTbl_P;
                        else
                                flag = 1;
                }
        }
        return NULL;
}

qaTbl_data* qaList_get(list_T *qaList,int ifIndex ,int qaIndex,int exact)
{
        struct list_node *np;
        qaTbl_data* qaTbl_P;
        int flag = 0;
        for ( np = LIST_FIRST ( qaList ); np != NULL; np = np->next ) {
                qaTbl_P = (qaTbl_data*) np->data;
                if(flag)
                        return qaTbl_P;
                if((qaTbl_P->ifIndex == ifIndex)&&(qaTbl_P->QAPEDCATableIndex == qaIndex)) {
                        if(exact)
                                return qaTbl_P;
                        else
                                flag = 1;
                }
        }
        return NULL;
}


opTbl_data* opList_get(list_T *opList,int index,int exact)
{
        struct list_node *np;
        opTbl_data* opTbl_P;
        int flag = 0;
        for ( np = LIST_FIRST ( opList ); np != NULL; np = np->next ) {
                opTbl_P = (opTbl_data*) np->data;
                if(flag)
                        return opTbl_P;
                if(opTbl_P->ifIndex == index) {
                        if(exact)
                                return opTbl_P;
                        else
                                flag = 1;
                }
        }
        return NULL;
}

phyhtTbl_data* PhyHTList_get(list_T *phyhtlist,int index,int exact)
{
	list_node *np;
	phyhtTbl_data *npht;
	int flag = 0;
	for(np = LIST_FIRST ( phyhtlist ); np != NULL ;np = np->next )
	{
		npht = (phyhtTbl_data *) np->data;
		if(flag)
			return npht;
		if ( (npht->ifindex == index) )
		{
			if(exact)
				return npht;
			else
				flag = 1;
		}
	}
	return NULL;	
}

htTbl_data* HTList_get(list_T *htlist,int index,int exact)
{
	list_node *np;
	htTbl_data *nht;
	int flag = 0;
	for(np = LIST_FIRST ( htlist ); np != NULL ;np = np->next )
	{
		nht = (htTbl_data *) np->data;
		if(flag)
			return nht;
		if ( (nht->ifindex == index) )
		{
			if(exact)
				return nht;
			else
				flag = 1;
		}
	}
	return NULL;	
}



rtTbl_data* rtList_get(list_T *rtlist,int index1,int index2,int exact)
{
	list_node *np;
	rtTbl_data *nrt;
	int flag = 0;
	for(np = LIST_FIRST ( rtlist ); np != NULL ;np = np->next )
	{
		nrt = (rtTbl_data *) np->data;
		if(flag)
			return nrt;
		if ( (nrt->ifindex == index1) && (nrt->index == index2) )
		{
			if(exact)
				return nrt;
			else
				flag = 1;
		}
	}
	return NULL;
}
	

rrTbl_data* rrList_get(list_T *rrlist,int index1,int index2,int exact)
{
	list_node *np;
	rrTbl_data *nrr;
	int flag = 0;
	for(np = LIST_FIRST ( rrlist ); np != NULL ;np = np->next )
	{
		nrr = (rrTbl_data *) np->data;
		if(flag)
			return nrr;
		if ( (nrr->ifindex == index1) && (nrr->index == index2) )
		{
			if(exact)
				return nrr;
			else
				flag = 1;
		}
	}
	return NULL;
}

poTbl_data* PoList_get(list_T *polist,int index,int exact)
{
	list_node *np;
	poTbl_data *npo;
	int flag = 0;
	for(np = LIST_FIRST ( polist ); np != NULL ;np = np->next )
	{
		npo = (poTbl_data *) np->data;
		if(flag)
			return npo;
		if ( (npo->ifindex == index) )
		{
			if(exact)
				return npo;
			else
				flag = 1;
		}
	}
	return NULL;
}

mdcTbl_data* mdcList_get(list_T *mdclist,int index1,int index2,int exact)
{
	list_node *np;
	mdcTbl_data *nmd;
	int flag = 0;
	for(np = LIST_FIRST ( mdclist ); np != NULL ;np = np->next )
	{
		nmd = (mdcTbl_data *) np->data;
		if(flag)
			return nmd;
		if ( (nmd->ifindex == index1) && (nmd->index == index2) )
		{
			if(exact)
				return nmd;
			else
				flag = 1;
		}
	}
	return NULL;
}

prvTbl_data* PrvList_get(list_T *prvlist,int index,int exact)
{
	list_node *np;
	prvTbl_data *npr;
	int flag = 0;
	for(np = LIST_FIRST ( prvlist ); np != NULL ;np = np->next )
	{
		npr = (prvTbl_data *) np->data;
		if(flag)
			return npr;
		if ( (npr->ifindex == index) )
		{
			if(exact)
				return npr;
			else
				flag = 1;
		}
	}
	return NULL;
}

dfTbl_data* dfList_get(list_T *dflist,int index1,int index2,int exact)
{
	list_node *np;
	dfTbl_data *ndf;
	int flag = 0;
	for(np = LIST_FIRST ( dflist ); np != NULL ;np = np->next )
	{
		ndf = (dfTbl_data *) np->data;
		if(flag)
			return ndf;
		if ( (ndf->ifindex == index1) && (ndf->index == index2))
		{
			if(exact)
				return ndf;
			else
				flag = 1;
		}
	}
	return NULL;
}

aaTbl_data* aaList_get(list_T *aalist,int index1,int index2,int exact)
{
	list_node *np;
	aaTbl_data *aa;
	int flag = 0;
	for(np = LIST_FIRST ( aalist ); np != NULL ;np = np->next )
	{
		aa = (aaTbl_data *) np->data;
		if(flag)
			return aa;
		if ( (aa->ifindex == index1) && (aa->index == index2))
		{
			if(exact)
				return aa;
			else
				flag = 1;
		}
	}
	return NULL;
}

scTbl_data* ScList_get(list_T *sclist,int index,int exact)
{
	struct list_node *np;
	scTbl_data *sc;
	int flag = 0;
	for(np = LIST_FIRST(sclist); np != NULL ;np = np->next )
	{
		sc = (scTbl_data *) np->data;
		if(flag)
			return sc;
		if(sc->ifindex == index)
		{
			if(exact)
				return sc;
			else
				flag = 1;
		}
	}
	return NULL;
}

smTxtbl_data* smTxList_get(list_T *smTxList,int index1,int index2,int exact)
{
	struct list_node *np;
        smTxtbl_data *smt;
        int flag = 0;
        for(np = LIST_FIRST(smTxList); np != NULL ;np = np->next )
        {
                smt = (smTxtbl_data *) np->data;
                if(flag)
                        return smt;
                if(smt->ifIndex == index1 && smt->MCSTxIndex == index2)
                {
                        if(exact)
                                return smt;
                        else
                                flag = 1;
                }
        }
        return NULL;
}

smRxtbl_data* smRxList_get(list_T *smRxList,int index1,int index2,int exact)
{
	struct list_node *np;
        smRxtbl_data *smr;
        int flag = 0;
        for(np = LIST_FIRST(smRxList); np != NULL ;np = np->next )
        {
                smr = (smRxtbl_data *) np->data;
                if(flag)
                        return smr;
                if(smr->ifIndex == index1 && smr->MCSRxIndex == index2)
                {
                        if(exact)
                                return smr;
                        else
                                flag = 1;
                }
        }
        return NULL;
}

TxpowTbl_data* TxPowList_get(list_T *TxPowList,int index,int exact)
{
        struct list_node *np;
        TxpowTbl_data *pt;
        int flag = 0;
        for(np = LIST_FIRST(TxPowList); np != NULL ;np = np->next )
        {
                pt = (TxpowTbl_data *) np->data;
                if(flag)
                        return pt;
                if(pt->ifIndex == index)
                {
                        if(exact)
                                return pt;
                        else
                                flag = 1;
                }
        }
        return NULL;
}


