#include <string.h>
#include <stdlib.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#include "ieee80211_ioctl.h"
#include "ieee802dot11_info.h"
#include "iwlib.h"
#include "AuthenticationAlgo.h"
#include "DataRates.h"
#include "WEPDefaultKey.h"
#include "StationConfigTable.h"
#include "Privacy.h"
#include "PhyOperation.h"
#include "MultiDoamianCapability.h"
#include "PhyHT.h"
#include "HTStationConfig.h"
#include "OpTable.h"
#include "QAPEDCATable.h"
#include "QCTable.h"
#include "CntTable.h"
#include "MCSTxTable.h"
#include "MCSRxTable.h"
#include "PhyTxPower.h"

#define ERR -1
#define DEBUG_MSG(fmt,args...) printf(fmt,##args)
extern list_T nsc_list;
extern list_T aa_list;
extern list_T df_list;
extern list_T prv_list;
extern list_T mdc_list;
extern list_T po_list;
extern list_T rt_list;
extern list_T rr_list;
extern list_T ht_list;
extern list_T phyht_list;
extern list_T opList;
extern list_T qaList;
extern list_T qcList;
extern list_T cntList;
extern list_T smTxList;
extern list_T smRxList;
extern list_T TxPowList;

void loadTxPowList(int skfd,char * ifName)
{
	struct iwreq wrq;
        TxpowTbl_data TxpowTbl_Node;
        int rv;

        memset( &TxpowTbl_Node , 0, sizeof(TxpowTbl_Node) );

        TxpowTbl_Node.ifIndex = if_nametoindex ( ifName );
        if ( !TxpowTbl_Node.ifIndex ) {
                /*snmp_log(LOG_ERR,"SNMP %s - %s %s\n",
                "loadTxPowList()", ifName, "has no ifIndex" );*/
                return;
        }
        DEBUG_MSG("ifIndex = %u\n\n",TxpowTbl_Node.ifIndex);
	
	/*dot11CurrentTxPowerLevel */
	wrq.u.param.value = IEEE80211_PARAM_CUR_TX_POWER;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
		TxpowTbl_Node.CurrPowL = wrq.u.param.value;
	DEBUG_MSG("TxpowTbl_Node.CurrPowL = %u\n",TxpowTbl_Node.CurrPowL);

	/*addList*/
	addList(&TxPowList,(char*)&TxpowTbl_Node,sizeof(TxpowTbl_Node));	
}

void loadsmTxList(int skfd,char *ifName)
{
	struct iwreq wrq;
        smTxtbl_data smTxtbl_Node;
        int rv;

        memset( &smTxtbl_Node , 0, sizeof(smTxtbl_Node) );

        smTxtbl_Node.ifIndex = if_nametoindex ( ifName );
        if ( !smTxtbl_Node.ifIndex ) {
                /*snmp_log(LOG_ERR,"SNMP %s - %s %s\n",
                "loadsmTxList()", ifName, "has no ifIndex" );*/
                return;
        }
        DEBUG_MSG("ifIndex = %u\n\n",smTxtbl_Node.ifIndex);
	
	/*SupportedMCSTxIndex*/
	wrq.u.param.value = IEEE80211_PARAM_SUP_MCS_TX_INDEX;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		smTxtbl_Node.MCSTxIndex = wrq.u.param.value;
	DEBUG_MSG("\nSupportedMCSTxIndex = %u \n",smTxtbl_Node.MCSTxIndex);

	/*SupportedMCSTxValue*/
	wrq.u.param.value = IEEE80211_PARAM_SUP_MCS_TX_VAL;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                smTxtbl_Node.MCSTxValue = wrq.u.param.value;
        DEBUG_MSG("\nSupportedMCSTxValue = %u \n",smTxtbl_Node.MCSTxValue);

	/*addList*/
        addList(&smTxList,(char*)&smTxtbl_Node,sizeof(smTxtbl_Node));
}

void loadsmRxList(int skfd,char *ifName)
{
        struct iwreq wrq;
        smRxtbl_data smRxtbl_Node;
        int rv;

        memset( &smRxtbl_Node , 0, sizeof(smRxtbl_Node) );

        smRxtbl_Node.ifIndex = if_nametoindex ( ifName );
        if ( !smRxtbl_Node.ifIndex ) {
                /*snmp_log(LOG_ERR,"SNMP %s - %s %s\n",
                "loadsmRxList()", ifName, "has no ifIndex" );*/
                return;
        }
        DEBUG_MSG("ifIndex = %u\n\n",smRxtbl_Node.ifIndex);

        /*SupportedMCSRxIndex*/
	wrq.u.param.value = IEEE80211_PARAM_SUP_MCS_RX_INDEX;
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                smRxtbl_Node.MCSRxIndex = wrq.u.param.value;
        DEBUG_MSG("\nSupportedMCSRxIndex = %u \n",smRxtbl_Node.MCSRxIndex);

        /*SupportedMCSRxValue*/
	wrq.u.param.value = IEEE80211_PARAM_SUP_MCS_RX_VAL;
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                smRxtbl_Node.MCSRxValue = wrq.u.param.value;
        DEBUG_MSG("\nSupportedMCSRxValue = %u \n",smRxtbl_Node.MCSRxValue);

        /*addList*/
        addList(&smRxList,(char*)&smRxtbl_Node,sizeof(smRxtbl_Node));
}


void loadcntList(int skfd,char * ifName)
{
        struct iwreq wrq;
        cntTbl_data cntNode;
        int rv;

        memset( &cntNode , 0, sizeof(cntNode) );

        cntNode.ifIndex = if_nametoindex ( ifName );
        if ( !cntNode.ifIndex ) {
                return;
        }
        DEBUG_MSG("ifIndex = %u\n\n",cntNode.ifIndex);
        strncpy ( wrq.ifr_name, ifName, IFNAMSIZ );

        /*TransFragCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_FRAG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransFragCount = wrq.u.param.value;
//        DEBUG_MSG("TransFragCount = %u \n",cntNode.TransFragCount);
	
	/*McastTransFragCount*/
        wrq.u.param.value = IEEE80211_PARAM_MCAST_TR_FRAME_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.McastTransFragCount = wrq.u.param.value;
        //DEBUG_MSG("McastTransFragCount = %u \n",cntNode.McastTransFragCount);

        /*FailedCount*/
        wrq.u.param.value = IEEE80211_PARAM_FAILED_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.FailedCount = wrq.u.param.value;
        //DEBUG_MSG("FailedCount = %u\n",cntNode.FailedCount);

        /*RetryCount*/
        wrq.u.param.value = IEEE80211_PARAM_RETRY_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.RetryCount = wrq.u.param.value ;
       // DEBUG_MSG("RetryCount = %u\n",cntNode.RetryCount);

        /*MultipleRetryCount*/
        wrq.u.param.value = IEEE80211_PARAM_MULTIPLE_RETRY_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.MultipleRetryCount = wrq.u.param.value ;
       // DEBUG_MSG("MultipleRetryCount = %u\n",cntNode.MultipleRetryCount);

	/*FrameDuplicateCount*/
        wrq.u.param.value = IEEE80211_PARAM_FRAME_DUP_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.FrameDuplicateCount = wrq.u.param.value ;
       // DEBUG_MSG("FrameDuplicateCount = %u\n",cntNode.FrameDuplicateCount);

        /*RTSSuccessCount*/
        wrq.u.param.value = IEEE80211_PARAM_RTS_SUCCESS_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.RTSSuccessCount = wrq.u.param.value ;
       // DEBUG_MSG("RTSSuccessCount = %u\n",cntNode.RTSSuccessCount);

        /*RTSFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_RTS_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.RTSFailureCount = wrq.u.param.value;
       // DEBUG_MSG("RTSFailureCount = %u\n",cntNode.RTSFailureCount);

        /*ACKFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_ACK_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.ACKFailureCount = wrq.u.param.value ;
       // DEBUG_MSG("ACKFailureCount = %u \n",cntNode.ACKFailureCount);

        /*RcvdFragmentCount*/
        wrq.u.param.value = IEEE80211_PARAM_RCV_FRAG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.RcvdFragmentCount = wrq.u.param.value ;
//	DEBUG_MSG("RcvdFragmentCount = %u\n",cntNode.RcvdFragmentCount);

        /*McastRcvdFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_MCAST_RCV_FRAME_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.McastRcvdFrameCount = wrq.u.param.value ;
  //      DEBUG_MSG("McastRcvdFrameCount = %u \n",cntNode.McastRcvdFrameCount);

        /*FCSErrorCount*/
        wrq.u.param.value = IEEE80211_PARAM_FCS_ERROR_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.FCSErrorCount = wrq.u.param.value ;
    //    DEBUG_MSG("FCSErrorCount = %u\n",cntNode.FCSErrorCount);

        /*TransFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_FRAME_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransFrameCount = wrq.u.param.value ;
      //  DEBUG_MSG("TransFrameCount = %u\n",cntNode.TransFrameCount);

	/*WEPUndecryptableCount*/
        wrq.u.param.value = IEEE80211_PARAM_WEP_UNDECRYPT_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.WEPUndecryptableCount = wrq.u.param.value ;
        //DEBUG_MSG("WEPUndecryptableCount = %u\n",cntNode.WEPUndecryptableCount);


        /*TransAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransAMSDUCount = wrq.u.param.value;
   //     DEBUG_MSG("TransAMSDUCount = %u\n",cntNode.TransAMSDUCount);

        /*FailedAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_FAIL_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.FailedAMSDUCount = wrq.u.param.value ;
     //   DEBUG_MSG("FailedAMSDUCount = %u\n",cntNode.FailedAMSDUCount);

        /*RetryAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_RETRY_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.RetryAMSDUCount = wrq.u.param.value ;
      //  DEBUG_MSG("RetryAMSDUCount = %u\n",cntNode.RetryAMSDUCount);

	/*MRetryAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_MUL_RETRY_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.MRetryAMSDUCount = wrq.u.param.value ;
        //DEBUG_MSG("MRetryAMSDUCount = %u\n",cntNode.MRetryAMSDUCount);

        /*TransOctInAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_OCTETS_INAMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.TransOctInAMSDUCount = wrq.u.param.value ;
     //   DEBUG_MSG("TransOctInAMSDUCount = %lu\n",cntNode.TransOctInAMSDUCount);

        /*AMSDUAckFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_ACKFAIL_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.AMSDUAckFailureCount = wrq.u.param.value ;
       // DEBUG_MSG("AMSDUAckFailureCount = %u\n",cntNode.AMSDUAckFailureCount);

        /*RcvdAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_RCV_AMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.RcvdAMSDUCount = wrq.u.param.value ;
       // DEBUG_MSG("RcvdAMSDUCount = %u\n",cntNode.RcvdAMSDUCount);

	/*RcvdOctetsInAMSDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_RCV_OCTETS_INAMSDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.RcvdOctetsInAMSDUCount = wrq.u.param.value ;
       // DEBUG_MSG("RcvdOctetsInAMSDUCount = %lu\n",cntNode.RcvdOctetsInAMSDUCount);

        /*TransAMPDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_AMPDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransAMPDUCount = wrq.u.param.value ;
       // DEBUG_MSG("TransAMPDUCount = %u \n",cntNode.TransAMPDUCount);

        /*TransMPDUsInAMPDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_MPDU_INAMPDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransMPDUsInAMPDUCount = wrq.u.param.value ;
       // DEBUG_MSG("dot11CountersTable = %lu\n",cntNode.TransMPDUsInAMPDUCount);

        /*TransOctetsInAMPDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_OCTETS_INAMPDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.TransOctetsInAMPDUCount = wrq.u.param.value ;
       // DEBUG_MSG("TransOctetsInAMPDUCount = %lu\n",cntNode.TransOctetsInAMPDUCount);

	/*AMPDURcvdCount*/
        wrq.u.param.value = IEEE80211_PARAM_RCV_AMPDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.AMPDURcvdCount = wrq.u.param.value ;
       // DEBUG_MSG("AMPDURcvdCount = %u\n",cntNode.AMPDURcvdCount);

        /*MPDUInRcvdAMPDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_MPDU_IN_RCV_AMPDU_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.MPDUInRcvdAMPDUCount = wrq.u.param.value ;
       // DEBUG_MSG("MPDUInRcvdAMPDUCount = %u\n",cntNode.MPDUInRcvdAMPDUCount);

        /*RcvdOctetsInAMPDUCount*/
        wrq.u.param.value = IEEE80211_PARAM_RCV_OCTETS_INAMPDU_COUNT;
        if(ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.RcvdOctetsInAMPDUCount = wrq.u.param.value ;
       // DEBUG_MSG("RcvdOctetsInAMPDUCount = %lu\n",cntNode.RcvdOctetsInAMPDUCount);

        /*AMPDUDelmtrCRCErrorCount*/
        wrq.u.param.value = IEEE80211_PARAM_AMPDU_DELIM_CRCERROR_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.AMPDUDelmtrCRCErrorCount = wrq.u.param.value ;
       // DEBUG_MSG("AMPDUDelmtrCRCErrorCount = %u\n",cntNode.AMPDUDelmtrCRCErrorCount);

	/*ImpBARFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_IMPLICIT_BAR_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.ImpBARFailureCount = wrq.u.param.value ;
       // DEBUG_MSG("ImpBARFailureCount = %u\n",cntNode.ImpBARFailureCount);

        /*ExpBARFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_EXPLICIT_BAR_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.ExpBARFailureCount = wrq.u.param.value ;
       // DEBUG_MSG("ExpBARFailureCount = %u\n",cntNode.ExpBARFailureCount);

        /*ChannelWidthSwitchCount*/
        wrq.u.param.value = IEEE80211_PARAM_CHANWIDTH_SWITCH_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.ChannelWidthSwitchCount = wrq.u.param.value ;
       // DEBUG_MSG("ChannelWidthSwitchCount = %u\n",cntNode.ChannelWidthSwitchCount);

        /*TwentyMHzTransFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_TWENTYMHZ_FRAME_TR_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.TwentyMHzTransFrameCount = wrq.u.param.value ;
      //  DEBUG_MSG("TwentyMHzTransFrameCount = %u\n",cntNode.TwentyMHzTransFrameCount);

	 /*FortyMHzTransFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_FORTYMHZ_FRAME_TR_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.FortyMHzTransFrameCount = wrq.u.param.value ;
       // DEBUG_MSG("FortyMHzTransFrameCount = %u\n",cntNode.FortyMHzTransFrameCount);

        /*TwentyMHzRcvdFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_TWENTYMHZ_FRAME_RCV_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TwentyMHzRcvdFrameCount = wrq.u.param.value ;
       // DEBUG_MSG("TwentyMHzRcvdFrameCount = %u\n",cntNode.TwentyMHzRcvdFrameCount);

        /*FortyMHzRcvdFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_FORTTYMHZ_FRAME_RCV_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.FortyMHzRcvdFrameCount = wrq.u.param.value ;
       // DEBUG_MSG("FortyMHzRcvdFrameCount = %u\n",cntNode.FortyMHzRcvdFrameCount);

	/*PSMPSuccessCount*/
        wrq.u.param.value = IEEE80211_PARAM_PSMP_SUCCESS_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.PSMPSuccessCount = wrq.u.param.value ;
       // DEBUG_MSG("PSMPSuccessCount = %u\n",cntNode.PSMPSuccessCount);


        /*PSMPFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_PSMP_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.PSMPFailureCount = wrq.u.param.value ;
    //    DEBUG_MSG("PSMPFailureCount = %u\n",cntNode.PSMPFailureCount);

        /*GrantedRDGUsedCount*/
        wrq.u.param.value = IEEE80211_PARAM_GRANTED_RDG_USED_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.GrantedRDGUsedCount = wrq.u.param.value ;
      //  DEBUG_MSG("GrantedRDGUsedCount = %u\n",cntNode.GrantedRDGUsedCount);

        /*GrantedRDGUnusedCount*/
        wrq.u.param.value = IEEE80211_PARAM_GRANTED_RDG_UNUSED_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                cntNode.GrantedRDGUnusedCount = wrq.u.param.value ;
       // DEBUG_MSG("GrantedRDGUnusedCount = %u\n",cntNode.GrantedRDGUnusedCount);
	
	/*TransFramesInGrantedRDGCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_FRAME_GRANTED_RDG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransFramesInGrantedRDGCount = wrq.u.param.value ;
       // DEBUG_MSG("TransFramesInGrantedRDGCount = %u\n",cntNode.TransFramesInGrantedRDGCount);

        /*TransOctetsInGrantedRDGCount*/
        wrq.u.param.value = IEEE80211_PARAM_TR_OCTETS_IN_GRANTED_RDG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.TransOctetsInGrantedRDGCount = wrq.u.param.value ;
       // DEBUG_MSG("TransOctetsInGrantedRDGCount = %lu\n",cntNode.TransOctetsInGrantedRDGCount);

        /*DualCTSSuccessCount*/
        wrq.u.param.value = IEEE80211_PARAM_DUAL_CTS_SUCCESS_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.DualCTSSuccessCount = wrq.u.param.value ;
       // DEBUG_MSG("DualCTSSuccessCount = %u\n",cntNode.DualCTSSuccessCount);

        /*DualCTSFailureCount*/
        wrq.u.param.value = IEEE80211_PARAM_DUAL_CTS_FAILURE_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                cntNode.DualCTSFailureCount = wrq.u.param.value ;
      //  DEBUG_MSG("DualCTSFailureCount = %u\n",cntNode.DualCTSFailureCount);

        /*addList*/
        addList(&cntList,(char*)&cntNode,sizeof(cntNode));
}
	


void loadqcList(int skfd,char * ifName)
{
        struct iwreq wrq;
        qcTbl_data qcNode;
        int rv,TID;

        memset( &qcNode , 0, sizeof(qcNode) );
        strncpy ( wrq.ifr_name, ifName, IFNAMSIZ );

        qcNode.ifIndex = if_nametoindex ( ifName );
        if ( !qcNode.ifIndex ) {
                /*snmp_log(LOG_ERR,"SNMP %s - %s %s\n",
                "loadqcList()", ifName, "has no ifIndex" );*/
                return;
        }

        /*QosCountersIndex*/
        qcNode.QosCountersIndex = 1;

        /*QosTransFragCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_TR_FRAG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0 )
                qcNode.QosTransFragCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosTransFragCount %u",qcNode.QosTransFragCount);

        /*QosFailedCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_FAILED_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosFailedCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosFailedCount %u",qcNode.QosFailedCount);
	
	 /*QosRetryCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_RETRY_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRetryCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRetryCount %u",qcNode.QosRetryCount);

        /*QosMultiRetryCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_MULTIPLE_RETRY_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosMultiRetryCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosMultiRetryCount %u",qcNode.QosMultiRetryCount);

        /*QosFrameDupCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_FRAME_DUP_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosFrameDupCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosFrameDupCount %u",qcNode.QosFrameDupCount);

        /*QosRTSSuccCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_RTS_SUCCESS_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRTSSuccCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRTSSuccCount %u",qcNode.QosRTSSuccCount);

        /*QosRTSFailCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_RTS_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRTSFailCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRTSFailCount %u",qcNode.QosRTSFailCount);

	/*QosRTSFailCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_RTS_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRTSFailCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRTSFailCount %u",qcNode.QosRTSFailCount);

        /*QosACKFailCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_ACK_FAIL_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosACKFailCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosACKFailCount %u",qcNode.QosACKFailCount);

        /*QosRcvdFragCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_RCV_FRAG_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRcvdFragCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRcvdFragCount %u",qcNode.QosRcvdFragCount);

        /*QosTransFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_TR_FRAME_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosTransFrameCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosTransFrameCount %u",qcNode.QosTransFrameCount);

        /*QosDiscardedFrameCount*/
        wrq.u.param.value = IEEE80211_PARAM_QOS_DISCARD_FRAME_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosDiscardedFrameCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosDiscardedFrameCount %u",qcNode.QosDiscardedFrameCount);
	
	/*QosMPDUsRcvdCount */
	wrq.u.param.value = IEEE80211_PARAM_QOS_MPDU_RCV_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosMPDUsRcvdCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosMPDUsRcvdCount %u",qcNode.QosMPDUsRcvdCount);

        /*QosRetriesRcvdCount */
	wrq.u.param.value = IEEE80211_PARAM_QOS_RETRY_RCV_COUNT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETCOUNTER,&wrq) >= 0)
                qcNode.QosRetriesRcvdCount = wrq.u.param.value ;
	DEBUG_MSG("\nQosRetriesRcvdCount %u",qcNode.QosRetriesRcvdCount);

        addList(&qcList,(char*)&qcNode,sizeof(qcNode));

}



void loadqaList(int skfd,char * ifName)
{
        struct iwreq wrq;
        qaTbl_data qaN1,qaN2,qaN3,qaN4;
        int rv,ifIndex;
        long aCWmax,aCWmin;
        struct qapIndex_Table qaindex;
        //union index_value u;
        memset( &qaindex, 0, sizeof(qaindex));

        memset( &qaN1 , 0, sizeof(qaN1) );
        memset( &qaN2 , 0, sizeof(qaN2) );
        memset( &qaN3 , 0, sizeof(qaN3) );
        memset( &qaN4 , 0, sizeof(qaN4) );
        strncpy ( wrq.ifr_name, ifName, IFNAMSIZ );

        ifIndex = if_nametoindex ( ifName );
        if ( !ifIndex ) {
             //snmp_log(LOG_ERR,"SNMP %s - %s %s\n", "loadqaList()", ifName, "has no ifIndex" );
                return;
        }
        qaN1.ifIndex = ifIndex;
        qaN2.ifIndex = ifIndex;
        qaN3.ifIndex = ifIndex;
	qaN4.ifIndex = ifIndex;

        /*QAPEDCATableIndex*/
        qaN1.QAPEDCATableIndex = 1;     /*if the value of the AC is AC_BK */
        qaN2.QAPEDCATableIndex = 2;     /*if the value of the AC is AC_BE */
        qaN3.QAPEDCATableIndex = 3;     /*if the value of the AC is AC_BE */
        qaN4.QAPEDCATableIndex = 4;     /*if the value of the AC is AC_VO */

        /*QAPEDCATableCWmin*/
        qaindex.subioctl_num = IEEE80211_WMMPARAMS_CWMIN;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN1.QAPEDCATableCWmin = wrq.u.param.value;
        //else
          // qaN1.QAPEDCATableCWmin = aCWmin;      /*Default Value of Index 1*/

        qaindex.index = 1;
        //wrq.u.name = (char *)&qaindex;
         memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)    
           qaN2.QAPEDCATableCWmin = wrq.u.param.value;
        //else
          // qaN2.QAPEDCATableCWmin = aCWmin;      /*Default Value of Index 2*/
  
        qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
	 memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN3.QAPEDCATableCWmin = wrq.u.param.value;
        //else
         //  qaN3.QAPEDCATableCWmin = ((aCWmin + 1)/2) - 1;      /*Default Value of Index 3*/
       
        qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
	 memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN4.QAPEDCATableCWmin = wrq.u.param.value;
        //else
          // qaN4.QAPEDCATableCWmin = ((aCWmin + 1)/4) - 1;      /*Default Value of Index 4*/

#if 0
        /*QAPEDCATableCWmin*/
        wrq.u.param.value = IEEE80211_WMMPARAMS_CWMIN; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) {
                aCWmin = wrq.u.param.value;
                qaN1.QAPEDCATableCWmin = aCWmin;
                qaN2.QAPEDCATableCWmin = aCWmin;
                qaN3.QAPEDCATableCWmin = ((aCWmin + 1)/2) - 1;
                qaN4.QAPEDCATableCWmin = ((aCWmin + 1)/4) - 1;
        }
#endif
        /*QAPEDCATableCWmax*/
       
        qaindex.subioctl_num = IEEE80211_WMMPARAMS_CWMAX;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
	 memcpy(&wrq.u.name, &qaindex, 8);	
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) 
            qaN1.QAPEDCATableCWmax = wrq.u.param.value;
       // else
         //   qaN1.QAPEDCATableCWmax = aCWmin;      /*Default Value of Index 1*/

        qaindex.index = 1;
       // wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN2.QAPEDCATableCWmax = wrq.u.param.value;
       // else
         //  qaN2.QAPEDCATableCWmax = aCWmin;      /*Default Value of Index 2*/

        qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN3.QAPEDCATableCWmax = wrq.u.param.value;
       // else
         //  qaN3.QAPEDCATableCWmax = ((aCWmin + 1)/2) - 1;      /*Default Value of Index 3*/

        qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if(ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN4.QAPEDCATableCWmax = wrq.u.param.value;
       // else
         //  qaN4.QAPEDCATableCWmax = ((aCWmin + 1)/4) - 1;      /*Default Value of Index 4*/


#if 0
        wrq.u.param.value = IEEE80211_WMMPARAMS_CWMAX; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) {
                aCWmax = wrq.u.param.value;
                qaN1.QAPEDCATableCWmax = aCWmax;
		qaN2.QAPEDCATableCWmax = 4*(aCWmin+1) - 1;
                qaN3.QAPEDCATableCWmax = aCWmin;
                qaN4.QAPEDCATableCWmax = (aCWmin+1)/2 - 1;
        }
#endif
        /*QAPEDCATableAIFSN*/

        qaindex.subioctl_num = IEEE80211_WMMPARAMS_AIFS;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
            qaN1.QAPEDCATableAIFSN = wrq.u.param.value;
        //else
          //  qaN1.QAPEDCATableAIFSN = 7;

        qaindex.index = 1;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN2.QAPEDCATableAIFSN = wrq.u.param.value;
        //else
          // qaN2.QAPEDCATableAIFSN = 3;

        qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN3.QAPEDCATableAIFSN = wrq.u.param.value;
        //else
         //  qaN3.QAPEDCATableAIFSN = 1;
    
        qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
 	memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
           qaN4.QAPEDCATableAIFSN = wrq.u.param.value;
        //else
          // qaN4.QAPEDCATableAIFSN = 1;

#if 0       
        wrq.u.param.value = IEEE80211_WMMPARAMS_AIFS; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) {
                qaN1.QAPEDCATableAIFSN = 7;
                qaN2.QAPEDCATableAIFSN = 3;
                qaN3.QAPEDCATableAIFSN = 1;
                qaN4.QAPEDCATableAIFSN = 1;
        }
#endif 
        /*QAPEDCATableTXOPLmt*/

        qaindex.subioctl_num = IEEE80211_WMMPARAMS_TXOP_MAX;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
            qaN1.QAPEDCATableTXOPLmt = wrq.u.param.value;
        //else
          //  qaN1.QAPEDCATableTXOPLmt = 0;

        qaindex.index = 1;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);	
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
            qaN2.QAPEDCATableTXOPLmt = wrq.u.param.value;
       // else
         //   qaN2.QAPEDCATableTXOPLmt = 0;


        qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
	 memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
            qaN3.QAPEDCATableTXOPLmt = wrq.u.param.value;
        //else
          //  qaN3.QAPEDCATableTXOPLmt = 3008;

        qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
	 memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
            qaN4.QAPEDCATableTXOPLmt = wrq.u.param.value;
        //else
          //  qaN4.QAPEDCATableTXOPLmt = 1504;

#if 0
        wrq.u.param.value = IEEE80211_WMMPARAMS_TXOP_MAX; 
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) {
                qaN1.QAPEDCATableTXOPLmt = 0;
                qaN2.QAPEDCATableTXOPLmt = 0;
                /* 3008 microseconds for IEEE 802.11a/g PHY and
                 * 6016 microseconds for IEEE 802.11b PHY ,
                 * if dot11QAPEDCATableIndex is 3
                 */
		qaN3.QAPEDCATableTXOPLmt = 3008;

                /* 1504 microseconds for IEEE 802.11a/g PHY and
                 * 3264 microseconds for IEEE 802.11b PHY,
                 * if dot11QAPEDCATableIndex is 4
                 */
                qaN4.QAPEDCATableTXOPLmt = 1504;
        }
#endif
	/*QAPEDCATableMSDULtime*/
        qaindex.subioctl_num = IEEE80211_WMMPARAMS_MSDU_LIFETIME;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
               qaN1.QAPEDCATableMSDULtime = wrq.u.param.value;
       // else 
         //      qaN1.QAPEDCATableMSDULtime = 500;


        qaindex.index = 1;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
               qaN2.QAPEDCATableMSDULtime = wrq.u.param.value;
        //else
          //     qaN2.QAPEDCATableMSDULtime = 500;
 
	qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
        memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
               qaN3.QAPEDCATableMSDULtime = wrq.u.param.value;
       // else
         //     qaN3.QAPEDCATableMSDULtime = 500;

	qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0)
               qaN4.QAPEDCATableMSDULtime = wrq.u.param.value;
       // else
         //      qaN4.QAPEDCATableMSDULtime = 500;



#if 0 
        wrq.u.param.value = IEEE80211_WMMPARAMS_MSDU_LIFETIME; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETWMMPARAMS, &wrq) >= 0) {
                qaN1.QAPEDCATableMSDULtime = wrq.u.param.value;
                qaN2.QAPEDCATableMSDULtime = wrq.u.param.value;
                qaN3.QAPEDCATableMSDULtime = wrq.u.param.value;
                qaN4.QAPEDCATableMSDULtime = wrq.u.param.value;
        }
#endif
        /*QAPEDCATableMandatory*/
        /*Always returns fale as we do not support admission control.*/
#if 0   
        qaindex.subioctl_num = IEEE80211_WMMPARAMS_AC_MANDATORY;
        qaindex.index = 0;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if((skfd, IEEE80211_IOCTL_GETWMMPARAMS, wrq) >= 0) 
            qaN1.QAPEDCATableMandatory = wrq.u.param.value;
        else
            qaN1.QAPEDCATableMandatory = 2;

        qaindex.index = 1;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if((skfd, IEEE80211_IOCTL_GETWMMPARAMS, wrq) >= 0)
            qaN2.QAPEDCATableMandatory = wrq.u.param.value;
        else
            qaN2.QAPEDCATableMandatory = 2;
 
        qaindex.index = 2;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if((skfd, IEEE80211_IOCTL_GETWMMPARAMS, wrq) >= 0)
            qaN3.QAPEDCATableMandatory = wrq.u.param.value;
        else
            qaN3.QAPEDCATableMandatory = 2;

        qaindex.index = 3;
        //wrq.u.name = (char *)&qaindex;
	memcpy(&wrq.u.name, &qaindex, 8);
        if((skfd, IEEE80211_IOCTL_GETWMMPARAMS, wrq) >= 0)
            qaN4.QAPEDCATableMandatory = wrq.u.param.value;
        else
            qaN4.QAPEDCATableMandatory = 2;
#endif


        qaN1.QAPEDCATableMandatory = 2;
        qaN2.QAPEDCATableMandatory = 2;
        qaN3.QAPEDCATableMandatory = 2;
        qaN4.QAPEDCATableMandatory = 2;


        addList(&qaList,(char*)&qaN1,sizeof(qaN1));
        addList(&qaList,(char*)&qaN2,sizeof(qaN2));
        addList(&qaList,(char*)&qaN3,sizeof(qaN3));
	addList(&qaList,(char*)&qaN4,sizeof(qaN4));
}


void loadopList(int skfd,char * ifName)
{
	struct iwreq wrq;
        struct ifreq ifr;
        iwrange range;
        char  buffer[sizeof ( iwrange ) * 2];
        int rv,i;
        opTbl_data opNode;
	char manf_id[2*SNMP_STR_LEN + 1] = {0};
	char prod_id[2*SNMP_STR_LEN + 1] = {0};
        
	memset( &opNode , 0, sizeof(opNode) );
        opNode.ifIndex = if_nametoindex ( ifName );
	DEBUG_MSG("ifindex %d\n",opNode.ifIndex);
        if ( !opNode.ifIndex ) {
               //snmp_log(LOG_ERR,"SNMP %s - %s %s\n","loadopList()", ifName, "has no ifIndex" );
                return;
        }

        strcpy ( ifr.ifr_name, ifName );
	memset(opNode.MACAddress,0,sizeof(opNode.MACAddress));
        rv = ioctl ( skfd, SIOCGIFHWADDR, &ifr );
        if ( rv >= 0 ) {
                /*sprintf ( opNode.MACAddress, "%02X:%02X:%02X:%02X:%02X:%02X\0",
                 ( u_char ) ifr.ifr_hwaddr.sa_data[0], ( u_char ) ifr.ifr_hwaddr.sa_data[1],
                 ( u_char ) ifr.ifr_hwaddr.sa_data[2], ( u_char ) ifr.ifr_hwaddr.sa_data[3],
                 ( u_char ) ifr.ifr_hwaddr.sa_data[4], ( u_char ) ifr.ifr_hwaddr.sa_data[5] );*/
                DEBUG_MSG ( "%02X:%02X:%02X:%02X:%02X:%02X\0",
                 ( u_char ) ifr.ifr_hwaddr.sa_data[0], ( u_char ) ifr.ifr_hwaddr.sa_data[1],
                 ( u_char ) ifr.ifr_hwaddr.sa_data[2], ( u_char ) ifr.ifr_hwaddr.sa_data[3],
                 ( u_char ) ifr.ifr_hwaddr.sa_data[4], ( u_char ) ifr.ifr_hwaddr.sa_data[5] );
		for(i=0;i<6;i++)
                	opNode.MACAddress[i] = ( u_char ) ifr.ifr_hwaddr.sa_data[i];
	}
	DEBUG_MSG(" MACAddress %s\n",opNode.MACAddress);
        
	strncpy ( wrq.ifr_name, ifName, IFNAMSIZ );
        /*RTSThreshold*/
        memset ( buffer, 0, sizeof ( buffer ));
        wrq.u.data.pointer = ( caddr_t ) &buffer;
        wrq.u.data.length = sizeof ( buffer );
        wrq.u.data.flags = 0;
        if ( ioctl ( skfd, SIOCGIWRTS, &wrq ) >= 0 ) {
                memcpy (( char * )&range, buffer, sizeof (iwrange ));
                opNode.RTSThreshold = range.max_rts;
        }
	DEBUG_MSG("RTSThreshold %u\n",opNode.RTSThreshold);

        /*shortRetryLimit */
        wrq.u.param.value = IEEE80211_PARAM_SHORT_RETRY_LT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
                opNode.shortRetryLimit = wrq.u.param.value;
	DEBUG_MSG("\nShortRetryLimit %u",opNode.shortRetryLimit);

        /*longRetryLimit */
        wrq.u.param.value = IEEE80211_PARAM_LONG_RETRY_LT; 
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.longRetryLimit = wrq.u.param.value;
	DEBUG_MSG("\nLongRetryLimit %u",opNode.longRetryLimit);

        /*fragmentationThreshold*/
        if( ioctl ( skfd, SIOCGIWFRAG, &wrq) >= 0)
         
         memcpy(&(opNode.fragmentationThreshold), &(wrq.u.frag), sizeof(iwparam));
	DEBUG_MSG("\nfragmentationThreshold %u\n",opNode.fragmentationThreshold);

        /*maxTransmitMSDULifetime*/
	 wrq.u.param.value = IEEE80211_PARAM_TX_TIMEOUT; 
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
                opNode.maxTransmitMSDULifetime = wrq.u.param.value;
	DEBUG_MSG("maxTransmitMSDULifetime %u\n",opNode.maxTransmitMSDULifetime);
        
	/*maxReceiveLifetime*/
        wrq.u.param.value = IEEE80211_PARAM_RX_TIMEOUT; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
                opNode.maxReceiveLifetime = wrq.u.param.value;
	DEBUG_MSG("maxReceiveLifetime %u\n",opNode.maxReceiveLifetime);

        /*manufacturerID*/
        wrq.u.data.flags = IEEE80211_PARAM_MANFACT_ID;
	wrq.u.data.pointer = (caddr_t)manf_id;
	wrq.u.data.length = SNMP_STR_LEN;
        if (ioctl(skfd, IEEE80211_IOCTL_GETSTRPARAM, &wrq) >= 0) {
		strncpy(opNode.manufacturerID,wrq.u.data.pointer,SNMP_STR_LEN);
		DEBUG_MSG("opNode.manufacturerID %s\n",opNode.manufacturerID);
	}
	else
		DEBUG_MSG("manufacturerID : IOCTL return ERR\n");

        /*productID*/
	memset(prod_id,0,(SNMP_STR_LEN+1));	
        wrq.u.data.flags = IEEE80211_PARAM_PRODUCT_ID; /* sub-ioctl no. */
	wrq.u.data.pointer = (caddr_t)prod_id;
	wrq.u.data.length = SNMP_STR_LEN + 1;
        if (ioctl(skfd, IEEE80211_IOCTL_GETSTRPARAM, &wrq) >= 0) {
		strncpy(opNode.productID,wrq.u.data.pointer,SNMP_STR_LEN);
		DEBUG_MSG("opNode.productID %s\n",opNode.productID);
	}
	else
		DEBUG_MSG("productID : IOCTL return ERR\n");
		
        
	/*HTOperatingMode*/
        wrq.u.param.value = IEEE80211_PARAM_HT_OPMODE; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.HTOperatingMode = wrq.u.param.value;
	DEBUG_MSG("HTOperatingMode %u\n",opNode.HTOperatingMode);

        /*RIFSMode*/
        wrq.u.param.value = IEEE80211_PARAM_RIFSMODE; 
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
	{
		if(wrq.u.param.value)
			opNode.RIFSMode = 1;
		else
			opNode.RIFSMode = 2;
	}
	DEBUG_MSG("\nRIFSMode %u",opNode.RIFSMode);
	
	/*PSMPControlledAccess*/
        wrq.u.param.value = IEEE80211_PARAM_PSMP_CTL_ACC; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
	{
		if(wrq.u.param.value)
                        opNode.PSMPControlledAccess = 1;
		else
                	opNode.PSMPControlledAccess = 2;
	}
	DEBUG_MSG("PSMPControlledAccess %u\n",opNode.PSMPControlledAccess);

        /*ServiceIntervalGranularity*/
        wrq.u.param.value = IEEE80211_PARAM_SER_INT_GRAN; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
                opNode.ServiceIntervalGranularity = wrq.u.param.value ;
	DEBUG_MSG("ServiceIntervalGranularity %u\n",opNode.ServiceIntervalGranularity);

        /*DualCTSProtection*/
        wrq.u.param.value = IEEE80211_PARAM_DUAL_CTS_PROT;
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
	{
                if(wrq.u.param.value)
	                opNode.DualCTSProtection = 1;
		else
	                opNode.DualCTSProtection = 2;
	}
	DEBUG_MSG("DualCTSProtection %u", opNode.DualCTSProtection);

        /*SigTXOPFullProtectionEnabled*/
        wrq.u.param.value = IEEE80211_PARAM_LSIGTXOPFP_EN; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
	{
                if(wrq.u.param.value)
                	opNode.SigTXOPFullProtectionEnabled = 1;
		else
                	opNode.SigTXOPFullProtectionEnabled = 2;
	}
	DEBUG_MSG("SigTXOPFullProtectionEnabled %u\n",opNode.SigTXOPFullProtectionEnabled);

        /*NonGFEntitiesPresent*/
        wrq.u.param.value = IEEE80211_PARAM_NON_GF_ENTITIES; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
	{
                if(wrq.u.param.value)
	                opNode.NonGFEntitiesPresent = 1;
		else
	                opNode.NonGFEntitiesPresent = 2;
	}
	DEBUG_MSG("NonGFEntitiesPresent %u\n",opNode.NonGFEntitiesPresent);

        /*PCOActivated*/
	 wrq.u.param.value = IEEE80211_PARAM_PCO_EN; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
	{
                if(wrq.u.param.value)
	                opNode.PCOActivated = 1;
		else
	                opNode.PCOActivated = 2;
	}
	DEBUG_MSG("PCOActivated %u\n",opNode.PCOActivated);

        /*PCO40MaxDuration*/
        wrq.u.param.value = IEEE80211_PARAM_PCO40_MAX_DUR; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.PCO40MaxDuration = wrq.u.param.value ;
	DEBUG_MSG("PCO40MaxDuration %u\n",opNode.PCO40MaxDuration);
        /*PCO20MaxDuration*/
        wrq.u.param.value = IEEE80211_PARAM_PCO20_MAX_DUR; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.PCO20MaxDuration = wrq.u.param.value ;
	DEBUG_MSG("PCO20MaxDuration %u\n",opNode.PCO20MaxDuration);

        /*PCO40MinDuration*/
        wrq.u.param.value = IEEE80211_PARAM_PCO40_MIN_DUR; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.PCO40MinDuration = wrq.u.param.value ;
	DEBUG_MSG("PCO40MinDuration %u\n",opNode.PCO40MinDuration);

        /*PCO20MinDuration*/
        wrq.u.param.value = IEEE80211_PARAM_PCO20_MIN_DUR; /* sub-ioctl no. */
        if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0)
                opNode.PCO20MinDuration = wrq.u.param.value ;
	DEBUG_MSG("PCO20MinDuration %u\n",opNode.PCO20MinDuration);

        /*addList*/
	printf("\n addr of opnaode 0x%x",&opNode);
        addList(&opList,(char*)&opNode,sizeof(opNode));
}



int load_phyhttbl (int skfd, char *ifname)
{
	phyhtTbl_data pht;
	struct iwreq wrq;
	
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);
	
	memset(&pht, 0, sizeof(pht));
	pht.ifindex = if_nametoindex( ifname );
	
	/*40MHZ Option Implemented*/
	wrq.u.param.value = IEEE80211_PARAM_40MHZ_OP_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
		if(wrq.u.param.value)
			pht.Mhz_opr_impl = 1;
		else
			pht.Mhz_opr_impl = 2;
	}
	DEBUG_MSG("\n40MHZ %u",pht.Mhz_opr_impl);

	/*40MHZ Option Enabled*/
	wrq.u.param.value = IEEE80211_PARAM_40MHZ_OP_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Mhz_opr_enb = 1;
		else
			pht.Mhz_opr_enb = 2;
	}
	DEBUG_MSG("\n40MHZ EN%u",pht.Mhz_opr_enb);

	/*Current Primary Channel*/
	wrq.u.param.value = IEEE80211_PARAM_CUR_PRIM_CHAN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.pri_ch = wrq.u.param.value;
	DEBUG_MSG("\nPrimary Channel %u",pht.pri_ch);

	/*Current Secondary Channel*/	
	wrq.u.param.value = IEEE80211_PARAM_CUR_SEC_CHAN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.sec_ch = wrq.u.param.value;
	DEBUG_MSG("\nSecondary Channel %u",pht.sec_ch);
	
	/*Number of Spatial Stream Implemented*/
	wrq.u.param.value = IEEE80211_PARAM_NUM_SPAT_STREAMS_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.Spatial_impl = wrq.u.param.value;
	DEBUG_MSG("\nNumber of Spatial Stream %u",pht.Spatial_impl);

	/*Number of Spatial Stream Enabled*/
	wrq.u.param.value = IEEE80211_PARAM_NUM_SPAT_STREAMS_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.Spatial_enb = wrq.u.param.value;
	DEBUG_MSG("\nNumber of Spatial Stream EN %u",pht.Spatial_enb);
	
	/*GreenField Option Implemented*/
	wrq.u.param.value = IEEE80211_PARAM_GREENFIELD_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
		if(wrq.u.param.value)
			pht.greenfield_impl = 1;
		else
			pht.greenfield_impl = 2;
	}
	DEBUG_MSG("\nGreenField Option Imp %u",pht.greenfield_impl);
		
	/*GreenField Option Enabled*/
	wrq.u.param.value = IEEE80211_PARAM_GREENFIELD_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.greenfield_enb = 1;
		else
			pht.greenfield_enb = 2;
	}		
	DEBUG_MSG("\nGreenField Option En %u",pht.greenfield_enb);

	/*ShortGIOption 20 IMPL*/	
	wrq.u.param.value = IEEE80211_PARAM_SHORTGI_IN_20_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.GIOption20_impl = 1;
		else
			pht.GIOption20_impl = 2;
	}
	DEBUG_MSG("\nShortGIOption 20 IM %u",pht.GIOption20_impl);

	/*ShortGIOption 20 EN*/	
	wrq.u.param.value = IEEE80211_PARAM_SHORTGI_IN_20_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.GIOption20_enb = 1;
		else
			pht.GIOption20_enb = 2;
	}	
	DEBUG_MSG("\nShortGIOption 20 EN %u",pht.GIOption20_enb);
	
	/*ShortGIOption 40 IMPL*/	
	wrq.u.param.value = IEEE80211_PARAM_SHORTGI_IN_40_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.GIOption40_impl = 1;
		else
			pht.GIOption40_impl = 2;
	}
	DEBUG_MSG("\nShortGIOption 40 IM %u",pht.GIOption40_impl);
	
	/*ShortGIOption 40 EN*/	
	wrq.u.param.value = IEEE80211_PARAM_SHORTGI_IN_40_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.GIOption40_enb = 1;
		else
			pht.GIOption40_enb = 2;
	}
	DEBUG_MSG("\nShortGIOption 40 EN %u",pht.GIOption40_enb);

	/*LDPC CodingOption IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_LDPC_CODING_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.LDPC_impl = 1;
		else
			pht.LDPC_impl = 2;
	}
	DEBUG_MSG("\nLDPC CodingOption IMP %u",pht.LDPC_impl);
	
	/*LDPC CodingOption ENB*/
	wrq.u.param.value = IEEE80211_PARAM_LDPC_CODING_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.LDPC_enb = 1;
		else
			pht.LDPC_enb = 2;
	}
	DEBUG_MSG("\nLDPC CodingOption ENB %u",pht.LDPC_enb);

	/*Dot11Tx STBC Option IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_TX_STBC_OPT_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Tx_STB_impl = 1;
		else
			pht.Tx_STB_impl = 2;
	}
	DEBUG_MSG("\nTX_STBC imp %u",pht.Tx_STB_impl);
		
	/*Dot11Tx STBC Option EN*/
	wrq.u.param.value = IEEE80211_PARAM_TX_STBC_OPT_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Tx_STB_enb = 1;
		else
			pht.Tx_STB_enb = 2;
	}
	DEBUG_MSG("\nTX-STBC EN %u",pht.Tx_STB_enb);
	
	/*Dot11Rx STBC Option IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_RX_STBC_OPT_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Rx_STB_impl = 1;
		else
			pht.Rx_STB_impl = 2;
	}
	DEBUG_MSG("\nRx STBC Option IM %u",pht.Rx_STB_impl);
		
	/*Dot11Rx STBC Option EN*/
	wrq.u.param.value = IEEE80211_PARAM_RX_STBC_OPT_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Rx_STB_enb = 1;
		else
			pht.Rx_STB_enb = 2;
	}
	DEBUG_MSG("\nRx STBC Option EN %u",pht.Rx_STB_enb);

	/*BeamForming Option IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_BEAMFORMING_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Beam_impl = 1;
		else
			pht.Beam_impl = 2;
	}
	DEBUG_MSG("\nBeamForming Option IM %u",pht.Beam_impl);
		
	/*BeamForming Option ENB*/
	wrq.u.param.value = IEEE80211_PARAM_BEAMFORMING_EN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Beam_enb = 1;
		else
			pht.Beam_enb = 2;
	}
	DEBUG_MSG("\nBeamForming Option EN %u",pht.Beam_enb);

	/*Highest Data Rate*/
	wrq.u.param.value = IEEE80211_PARAM_HIGHEST_SUP_DATA_RATE;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.Highest_datarate = wrq.u.param.value;		
	DEBUG_MSG("\nHighest Data Rate %u",pht.Highest_datarate);

	/*TXMCSSet Defined*/
	wrq.u.param.value = IEEE80211_PARAM_TX_MCSSET_DEFINED;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Tx_mcsset = 1;
		else
			pht.Tx_mcsset = 2;
	}
	DEBUG_MSG("\nTXMCSSet Defined %u",pht.Tx_mcsset);

	/*TXRXMCSSetNotEqual*/
	wrq.u.param.value = IEEE80211_PARAM_TXRX_MCSSET_NOTEQ;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Rx_mcsset = 1;
		else
			pht.Rx_mcsset = 2;
	}
	DEBUG_MSG("\nTXRXMCSSetNotEqual %u",pht.Rx_mcsset);
	
	/*Max Number of SpatialStreams Supported */
	wrq.u.param.value = IEEE80211_PARAM_TX_MAX_NO_SPAT_STREAMS_SUP;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		pht.max_Spatial = wrq.u.param.value;
	DEBUG_MSG("\nMax Number of SpatialStreams Sup %u",pht.max_Spatial);

	/*TxUnequal Modulation Supported*/
	wrq.u.param.value = IEEE80211_PARAM_TX_UNEQUAL_MOD_SUP;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			pht.Modulation_support = 1;
		else
			pht.Modulation_support = 2;
	}
	DEBUG_MSG("\nTxUnequal Modulation Supported %u",pht.Modulation_support);

	/*Addlist*/
	addList(&phyht_list,(char *)&pht,sizeof(pht));
}

int load_httbl (int skfd, char *ifname)	
{
	int i;
	htTbl_data ht;
	struct iwreq wrq;
	unsigned char op_mcsset[MCSSET_MAX_SIZE * 2];
	
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);

	memset(&ht, 0, sizeof(ht));
	ht.ifindex = if_nametoindex( ifname );
		
	/*HTOperational MCSSET String */
	memset(op_mcsset,0,( MCSSET_MAX_SIZE * 2) );
        wrq.u.data.pointer = (caddr_t) op_mcsset;
        wrq.u.data.length = MCSSET_MAX_SIZE ;
        wrq.u.data.flags = IEEE80211_PARAM_HT_OP_MCSSET;
        if(ioctl(skfd, IEEE80211_IOCTL_GETSTRPARAM,&wrq) >= 0)
        {
               	memcpy(ht.mcsset,op_mcsset,MCSSET_MAX_SIZE);
		for(i=0;i<strlen(op_mcsset);i++)
			DEBUG_MSG("\nHTOperational MCSSET %02X",ht.mcsset[i]);
		if(!strlen(ht.mcsset))
			DEBUG_MSG("\nHTOperational MCSSET :retrn NULL");
        }

	/*Power Save*/	
	wrq.u.param.value = IEEE80211_PARAM_MIMO_PWRSAVE;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		ht.pwr_save = wrq.u.param.value;
	DEBUG_MSG("\npwr_save %u", ht.pwr_save);
	
	/*Delay BLock ACK*/
	wrq.u.param.value = IEEE80211_PARAM_11N_DEL_BLKACK_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
		if(wrq.u.param.value)
			ht.delay_blkack = 1;
		else
			ht.delay_blkack = 2;
	}
	DEBUG_MSG("\ndely_blkack %u", ht.delay_blkack);	
	
	/*MAX AMSDU LENGTH*/
	wrq.u.param.value = IEEE80211_PARAM_MAX_AMSDU_LEN;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		ht.AMSDU_len = wrq.u.param.value;
	DEBUG_MSG("\nMAX AMSDU LENGTH = %u ",ht.AMSDU_len);
	
	/*PSMP Option*/
	wrq.u.param.value = IEEE80211_PARAM_PSMP_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
		/*ht.POption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			ht.POption = 1;
		else
			ht.POption = 2;
	}
	DEBUG_MSG("\nht.POption %u", ht.POption);	
	
	/*STBC */
	wrq.u.param.value = IEEE80211_PARAM_STBC_CF_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			ht.FrameOption = 1;
		else
			ht.FrameOption = 2;
	}		
	DEBUG_MSG("\nSTBC %u",ht.FrameOption);	
	
	/*LSIG TXOP*/	
	wrq.u.param.value = IEEE80211_PARAM_LSIG_TXOP_PROT_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			ht.ProtectOption = 1;
		else
			ht.ProtectOption = 2;
	}		
	DEBUG_MSG("\nLSIG TXOP %u",ht.ProtectOption);	

	/*RX AMPDU Factor */
	wrq.u.param.value = IEEE80211_PARAM_MAX_RX_AMPDU_FACTOR;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		ht.RxAMPDU_fac = wrq.u.param.value;
	DEBUG_MSG("\nRX AMPDU FACTOR = %u",ht.RxAMPDU_fac);
	
	/*MIN MPDU SPACE*/
	wrq.u.param.value = IEEE80211_PARAM_MIN_MPDU_ST_SPACING;	
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >=0 )
		ht.minMPDU = wrq.u.param.value;
	DEBUG_MSG("\nMIN MPDU SPACE %u",ht.minMPDU);	

	/*PCO Option*/	
	wrq.u.param.value = IEEE80211_PARAM_PCO_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			ht.PcoOption = 1;
		else
			ht.PcoOption = 2;
	}
	DEBUG_MSG("\nPCO Option %u",ht.PcoOption);	
	
	/*Transition Time*/
	wrq.u.param.value = IEEE80211_PARAM_CH_TRANS_TIME;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
		ht.transition_time = wrq.u.param.value;
	DEBUG_MSG("\nTransition Time %u",ht.transition_time);	

	/*FeedBack Option*/	
	wrq.u.param.value = IEEE80211_PARAM_MCS_FEEDBACK_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
			ht.feedbackOption = wrq.u.param.value;
	DEBUG_MSG("\nht.feedbackOption %u",ht.feedbackOption);	
	
	/*HTControl Field*/	
	wrq.u.param.value = IEEE80211_PARAM_HT_CF_SUPPORTED;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			ht.cntrl_field = 1;
		else
			ht.cntrl_field = 2;
	}		
	DEBUG_MSG("\nht.cntrl_field %u",ht.cntrl_field);	
	
	/*RDResponder Option*/	
	wrq.u.param.value = IEEE80211_PARAM_RD_RESPONDER_IM;
	if (ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >=0 )
	{
                if(wrq.u.param.value)
			ht.Rdresponder = 1;
		else
			ht.Rdresponder = 2;
	}		
	DEBUG_MSG( "\nht.Rdresponder %u\n",ht.Rdresponder);	

	/*Addlist*/
	addList(&ht_list,(char *)&ht,sizeof(ht));
}

int load_rrtbl (int skfd,char *ifname)
{
	rrTbl_data rr;
	struct iwreq wrq;
	int i=0;
	
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);

	memset(&rr, 0, sizeof(rr));
	rr.ifindex = if_nametoindex( ifname );
	for(i = 1;i <= 8; i++)
	{
		rr.index = i;
		wrq.u.param.flags = i;
	
		/*SupportedDataRatesRxValue*/	
		wrq.u.param.value = IEEE80211_PARAM_SUP_DATA_RATES_RX;
		if(ioctl (skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
		{
			rr.rxval = wrq.u.param.value;
		}
		/*Addlist*/
		addList(&rr_list,(char *)&rr,sizeof(rr));
	}
	return 0;
}

int load_rttbl (int skfd,char *ifname)
{
	rtTbl_data rt;
	struct iwreq wrq;
	int i=0;
	
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);

	memset(&rt, 0, sizeof(rt));
	rt.ifindex = if_nametoindex( ifname );
	for(i = 1;i <= 8; i++)
	{
		rt.index = i;
		wrq.u.param.flags = i;

		/*SupportedDataRatesTxValue*/
		wrq.u.param.value = IEEE80211_PARAM_SUP_DATA_RATES_TX;
		if(ioctl (skfd, IEEE80211_IOCTL_GETPARAM, &wrq) >= 0 )
		{
			rt.txval = wrq.u.param.value;
		}
		/*Addlist*/
		addList(&rt_list,(char *)&rt,sizeof(rt));
	}
	return 0;
}

int load_phyoprtbl (int skfd,char *ifname)
{
	poTbl_data po;
	struct iwreq wrq;
	char name[IFNAMSIZ + 1] = {0};	

	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);
		
	memset(&po, 0, sizeof(po));
	po.ifindex = if_nametoindex( ifname );
	
	if(ioctl (skfd,SIOCGIWNAME,&wrq) >= 0)
	{
		strncpy(name,wrq.u.name,IFNAMSIZ);
		name[IFNAMSIZ] = '\0';
	}
	
	if ( strstr ( name, "IEEE 802.11-FS" )) 
		po.phy_type = 1;
	if ( strstr ( name, "IEEE 802.11-DS" )) 
		po.phy_type = 2;
	if ( strstr ( name, "IEEE 802.11-IR" )) 
		po.phy_type = 3;
	if ( strstr ( name, "IEEE 802.11-OFDM" )) 
		po.phy_type = 4;
	if ( strstr ( name, "IEEE 802.11-OFDM/DS" )) 
		po.phy_type = 5;
	if ( strstr ( name, "IEEE 802.11-TURBO" )) 
		po.phy_type = 6;
        if ( strstr ( name, "IEEE 802.11-HT")) 
                po.phy_type = 7;	
	DEBUG_MSG("Name Phy Type %s %u\n",name,po.phy_type);
	
	wrq.u.param.value = IEEE80211_PARAM_CUR_REG_DOM;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0 )
		po.curr_domain = wrq.u.param.value;
	DEBUG_MSG("Curr dmn %u\n",po.curr_domain);
 
	wrq.u.param.value = IEEE80211_PARAM_PHY_TEMP_TYPE;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0 )
		po.temp_type = wrq.u.param.value;
	DEBUG_MSG("temp  %u\n",po.temp_type);

	/*Addlist*/ 
	addList(&po_list,(char *)&po,sizeof(po));
	
	return 0;
}

int load_mdctbl (int skfd,char *ifname)
{
	mdcTbl_data nmd;
	struct ifreq ifr;
	struct iwreq wrq;
	
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);
	
	memset(&nmd, 0, sizeof(nmd));
	
	nmd.ifindex = if_nametoindex( ifname );
	nmd.index = 1;

	wrq.u.param.value = IEEE80211_PARAM_FIRST_CHAN_NUM;
        if(ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                nmd.first_ch = wrq.u.param.value;
        DEBUG_MSG("first chan %u\n",nmd.first_ch);

        wrq.u.param.value = IEEE80211_PARAM_NUM_OF_CHANS;
        if(ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                nmd.num_ch = wrq.u.param.value;
        DEBUG_MSG("No. of Chan %u\n",nmd.num_ch);

        wrq.u.param.value = IEEE80211_PARAM_MAX_TR_POWLEVEL;
        if(ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
                nmd.max_pwr = wrq.u.param.value;
        DEBUG_MSG("Powerlvl %u\n",nmd.max_pwr);
	
	/*Addlist*/
	addList(&mdc_list,(char *)&nmd,sizeof(nmd));
}

int load_prvtbl (int skfd,char *ifname)
{
	prvTbl_data npr;
	struct ifreq ifr;
	struct iwreq wrq;
	unsigned char key[IW_ENCODING_TOKEN_MAX];
	int key_flags;
	int key_size;
	int wepcurr_key;
		
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);
	memset(&ifr, 0, sizeof(ifr));
	strcpy( ifr.ifr_name,ifname);

	memset(&npr,0,sizeof(npr));

	wrq.u.data.pointer = (caddr_t) &key;
	wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
	wrq.u.data.flags = 0;
	if(ioctl (skfd, SIOCGIWENCODE, &wrq) >= 0) {
		key_flags = wrq.u.data.flags;
		key_size = wrq.u.data.length;
		wepcurr_key = wrq.u.data.flags & IW_ENCODE_INDEX;
	}
	
	npr.ifindex = if_nametoindex( ifname );
	npr.priv_invoked = 2;	
	npr.unencrypted = 2;		
	npr.key_id = wepcurr_key;
	if(key_size != 0 && !(key_flags & IW_ENCODE_DISABLED))
		npr.priv_invoked = 1;	
	if(key_flags & IW_ENCODE_RESTRICTED)
		npr.unencrypted = 1;		
	
	DEBUG_MSG("\nPriv_inkd %d",npr.priv_invoked);
	DEBUG_MSG("\nPriv_keyid %d",npr.key_id);
	DEBUG_MSG("\nPriv_unencryt %d",npr.unencrypted);

	/* WEPICVErrorCount */	
	wrq.u.param.value = IEEE80211_PARAM_ICV_COUNT;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		npr.err_cnt = wrq.u.param.value;
	DEBUG_MSG("\n WEPICVErrorCount %u",npr.err_cnt);
	
	/* WEPExcludedCount */	
	wrq.u.param.value = IEEE80211_PARAM_WEP_EXC_COUNT;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		npr.exld_cnt = wrq.u.param.value;
	DEBUG_MSG("\nWEPExcludedCount %u ",npr.exld_cnt);

	/* RSNAEnabled */	
	wrq.u.param.value = IEEE80211_PARAM_WPA;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		if(wrq.u.param.value == 0 ) {
			npr.RSNA_enb = 2;
			DEBUG_MSG("\nWPA , WPA2 both are Disabled");
		}
		else if(wrq.u.param.value == 1 ) {
			npr.RSNA_enb = 1;
			DEBUG_MSG("\nWPA Enabled");
		}
		else if(wrq.u.param.value == 2 ) {
			npr.RSNA_enb = 1;
			DEBUG_MSG("\nWPA1 Enabled");
		}
		else if(wrq.u.param.value == 3 ) {
			npr.RSNA_enb = 1;
			DEBUG_MSG("\nWPA and WPA1 are Enabled");
		}			
	}
			

	/* RSNAPreauthenticationEnabled */	
	wrq.u.param.value = IEEE80211_PARAM_WPA;
	if(ioctl (skfd,IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*RSN is enabled which automatically enables preauthentication*/	
                if(wrq.u.param.value == 2)
			npr.RSNAPre_enb = 1;
		else
			npr.RSNAPre_enb = 2;
	}
	DEBUG_MSG("\nRSNAPreAuthEnabled %u",npr.RSNAPre_enb);

	/*Addlist*/
	addList(&prv_list,(char *)&npr,sizeof(npr));
	
	return 0;
	
}

int load_dftbl (int skfd,char *ifname)
{
	dfTbl_data ndf;
	int i;
	memset(&ndf, 0, sizeof(ndf));
	ndf.ifindex = if_nametoindex( ifname );
	for(i = 1; i <= 4; i++)
	{
		ndf.index = i;
		strcpy(ndf.key,"*****");
		/*Addlist*/		
		addList(&df_list,(char *)&ndf,sizeof(ndf));
	}
	return 0;	
}

int load_aatbl(int skfd,char *ifname)
{
	aaTbl_data naa , naa1;	
	struct iwreq wrq;
	unsigned char key[IW_ENCODING_TOKEN_MAX];
	int key_flags;
	int key_size;
	int has_key;
	
	memset(key, 0, IW_ENCODING_TOKEN_MAX);
	memset(&wrq, 0, sizeof(wrq));
	strncpy(wrq.ifr_name,ifname,IFNAMSIZ);
	
	wrq.u.data.pointer = (caddr_t) &key;
	wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
	wrq.u.data.flags = 0;
	if(ioctl (skfd, SIOCGIWENCODE, &wrq) >= 0) {
		key_flags = wrq.u.data.flags;
		key_size = wrq.u.data.length;
	}
	memset(&naa, 0, sizeof(naa));
	naa.ifindex = if_nametoindex( ifname );
	naa.index = 1;
       
        if(!(key_flags & IW_ENCODE_DISABLED))
         {
           naa.algo = 1;   /*Open Key*/
           if(key_flags & IW_ENCODE_OPEN )
	        naa.algo_enbl = 1; 	/*Enable by Default*/
	   else
                naa.algo_enbl = 2;      /*Disabled*/

          /*AddList*/
       	   addList(&aa_list,(char *)&naa,sizeof(naa));
	
	   naa.index = 2;
           naa.algo = 2;                /*Shared Key*/
           if(key_flags & IW_ENCODE_RESTRICTED)
	        naa.algo_enbl = 1; 	/*Enabled*/
           else
                naa.algo_enbl = 2;      /*Disabled*/
   
	/*AddList*/
	    addList(&aa_list,(char *)&naa,sizeof(naa));

	    return 0;	
         }

        else
          {
            naa.index = 1;
             naa.algo = 1;
             naa.algo_enbl = 2;
            addList(&aa_list, (char *)&naa, sizeof(naa)); 

            naa.index = 2;
             naa.algo = 2;
             naa.algo_enbl = 2;
            addList(&aa_list, (char *)&naa, sizeof(naa));
           }
} 
int load_sctbl(int skfd,char *ifname)
{
	scTbl_data nsc;
	struct ifreq ifr;
	struct iwreq wrq;
	struct iw_param iwp;
	char essid[IW_ESSID_MAX_SIZE + 1];
	iwrange iwr;
	char buffer[sizeof (iwrange) * 2],country[5] = {0};
	int rc,i=0,j=0;
	struct sockaddr	*sock;
	char buff[256];

	DEBUG_MSG("********************Interface name %s***********************",ifname);
	memset(&nsc, 0,sizeof(nsc));
	memset(&wrq, 0, sizeof(wrq));
	strcpy(wrq.ifr_name,ifname);
	memset(&ifr, 0, sizeof(ifr));
	strcpy( ifr.ifr_name,ifname);

	
	nsc.ifindex = if_nametoindex( ifname );
	DEBUG_MSG(" Ifindex = %d\n",nsc.ifindex);	
	/*Station ID*/
	strcpy(ifr.ifr_name,ifname);
	rc = ioctl( skfd, SIOCGIFHWADDR, &ifr);	
	if(rc >= 0)
	{
		/*sprintf(nsc.stationID, "%02X:%02X:%02X:%02X:%02X:%02X",
			(u_char) ifr.ifr_hwaddr.sa_data[0], (u_char) ifr.ifr_hwaddr.sa_data[1],
			(u_char) ifr.ifr_hwaddr.sa_data[2], (u_char) ifr.ifr_hwaddr.sa_data[3],
		(u_char) ifr.ifr_hwaddr.sa_data[4], (u_char) ifr.ifr_hwaddr.sa_data[5]);*/
		DEBUG_MSG("\n%02X:%02X:%02X:%02X:%02X:%02X",
			(u_char) ifr.ifr_hwaddr.sa_data[0], (u_char) ifr.ifr_hwaddr.sa_data[1],
			(u_char) ifr.ifr_hwaddr.sa_data[2], (u_char) ifr.ifr_hwaddr.sa_data[3],
			(u_char) ifr.ifr_hwaddr.sa_data[4], (u_char) ifr.ifr_hwaddr.sa_data[5]);
		for(i = 0;i < 6; i++)
			nsc.stationID[i] = (u_char) ifr.ifr_hwaddr.sa_data[i];
	}
	DEBUG_MSG("\nnsc.stationID = %s",nsc.stationID);
	
	/*Medium occupancy*/
	wrq.u.param.value = IEEE80211_PARAM_MED_OCC_LIM;
	rc = ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq);
	if(rc >= 0)
	{
		nsc.mediumOccupancyLimit = wrq.u.param.value;
	}
	DEBUG_MSG("\nmedium occupancy %u\n",nsc.mediumOccupancyLimit);

	/*CFPOLLABLE*/
	wrq.u.param.value = IEEE80211_PARAM_CFPOLLABLE;
	rc = ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq);
	if(rc >= 0)
	{
		/*nsc.CFPPollable = wrq.u.param.value;*/
		if(wrq.u.param.value)
			nsc.CFPPollable = 1;
		else
			nsc.CFPPollable = 2;
	}
	DEBUG_MSG("CFPOLLABLE %u\n",nsc.CFPPollable);
	
	/*CFPPERIOD*/
	wrq.u.param.value = IEEE80211_PARAM_CFPPERIOD;
	rc = ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq);
	if(rc >= 0)
	{
		nsc.CFPPeriod = wrq.u.param.value;
	}	
	DEBUG_MSG("CFPPERIOD %u\n",nsc.CFPPeriod);

	/*CFPMAXDuration*/
	wrq.u.param.value = IEEE80211_PARAM_CFPMAXPERIOD;
	rc = ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq);
	if(rc >= 0)
	{
		nsc.maxDuration = wrq.u.param.value;
	}
	DEBUG_MSG("CFPMAXDuration %u\n",nsc.maxDuration);

	/*Privacy Option Implemented */	
	wrq.u.param.value = IEEE80211_PARAM_PRIVACY;
	rc = ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq);
	if(rc >= 0)
	{
		nsc.privacyOptionImplemented = wrq.u.param.value;
		if(wrq.u.param.value)
			nsc.privacyOptionImplemented = 1;
		else
			nsc.privacyOptionImplemented = 2;
	}
	DEBUG_MSG("privacyOptionImplemented %u\n",nsc.privacyOptionImplemented);


	/*Power management Mode*/
	wrq.u.power.flags = 0;
	rc = ioctl (skfd,SIOCGIWPOWER,&wrq);
	if(rc >=0 )
	{
		memcpy(&iwp, &wrq.u.power, sizeof(struct iw_param));
		nsc.powerManagementMode = 1;
		if( !iwp.disabled && iwp.flags & IW_POWER_MIN )
			nsc.powerManagementMode = 2;
	}
	DEBUG_MSG("management %u\n",nsc.powerManagementMode);


	/*Get ESSID*/
	memset(essid, 0, (IW_ESSID_MAX_SIZE + 1));
	wrq.u.essid.pointer = (caddr_t)essid;		
	wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
	wrq.u.essid.flags = 0;
	if( ioctl (skfd,SIOCGIWESSID,&wrq) >=0 )
	{
		//	if(strlen (essid))
		strncpy(nsc.desiredSSID,(char *)essid, strlen(essid));
                   nsc.desiredSSID[strlen(essid)]= '\0';

 
	}
	DEBUG_MSG("essid %s\n",nsc.desiredSSID);

	/*Get BSS Type*/
	if(ioctl (skfd, SIOCGIWMODE, &wrq) >= 0)
	{
		if(wrq.u.mode == IW_MODE_ADHOC)
			nsc.desiredBSSType = 2;
		else if(ioctl (skfd, SIOCGIWAP, &wrq) >=0 )
			nsc.desiredBSSType = 1;
		else
			nsc.desiredBSSType = 3;
	}
	DEBUG_MSG("Bss Type %u\n",nsc.desiredBSSType);

	/*Operational RateSet*/
	memset(nsc.operationalRateSet , 0 , 256 );
	memset(buffer, 0 ,sizeof ( buffer) );
	wrq.u.data.pointer = (caddr_t) &buffer;
	wrq.u.data.length = sizeof(buffer);
	wrq.u.data.flags = 0;
	if ( ioctl(skfd, SIOCGIWRANGE, &wrq) >= 0)
	{
		memcpy((char *)&iwr, buffer, sizeof(iwrange));
		DEBUG_MSG("nsc.operationalRateSet\t");
		for(i = 0; i < iwr.num_bitrates && j < 256; i++)
		{
			nsc.operationalRateSet[j++] = (char) (iwr.bitrate[i] / 500000L );
			DEBUG_MSG("[%d] %d ",(j-1),nsc.operationalRateSet[j-1]);
		}
	}
	

	/*Beacon Period*/
	wrq.u.param.value = IEEE80211_PARAM_BEACON_INTERVAL;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.beaconPeriod = wrq.u.param.value;
	DEBUG_MSG("Beacon %u\n",nsc.beaconPeriod);

	/*DTM Period*/
	wrq.u.param.value = IEEE80211_PARAM_DTIM_PERIOD;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.DTIMPeriod = wrq.u.param.value;
	DEBUG_MSG("DTM Period %u\n",nsc.DTIMPeriod);

	/*Association response Time*/	
	wrq.u.param.value = IEEE80211_PARAM_INACT;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.associationResponseTimeOut = wrq.u.param.value;
	DEBUG_MSG("nsc.associationResponseTimeOut %u\n",nsc.associationResponseTimeOut);

	/*Disassociation reason*/
	wrq.u.param.value = IEEE80211_PARAM_DISASSOC_REASON;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.disAssociationReason = wrq.u.param.value;
	DEBUG_MSG("nsc.disAssociationReason %u\n",nsc.disAssociationReason);

	/*DisAssociation Station */
	memset(nsc.disAssociationStation, 0, (MACADDR_LEN + 1));
	wrq.u.data.flags = IEEE80211_PARAM_DISASSOC_STATION;
	wrq.u.data.pointer = (caddr_t)&buff;
        wrq.u.data.length = sizeof(buff);
	if( ioctl(skfd, IEEE80211_IOCTL_GETADDRPARAM,&wrq) >= 0)
	{
		sock = (struct sockaddr *) buff;	
		DEBUG_MSG(nsc.disAssociationStation, "%02X:%02X:%02X:%02X:%02X:%02X",
			(u_char)sock->sa_data[0],(u_char)sock->sa_data[1],(u_char)sock->sa_data[2],(u_char)sock->sa_data[3],(u_char)sock->sa_data[4],(u_char)sock->sa_data[5]);
		for(i = 0; i < 6; i++)
			nsc.disAssociationStation[i] = (u_char)sock->sa_data[i]; 
	}
	DEBUG_MSG("nsc.disAssociationStation %s\n",nsc.disAssociationStation);

	/*DeAuthenticate Reason*/
	wrq.u.param.value = IEEE80211_PARAM_DEAUTH_REASON;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.deAuthenticationReason = wrq.u.param.value;
	DEBUG_MSG("\nnsc.deAuthenticationReason %u\n",nsc.deAuthenticationReason);

	/*DeAuthenticate Station*/
	memset(nsc.deAuthenticationStation, 0, (MACADDR_LEN + 1));
	wrq.u.data.flags = IEEE80211_PARAM_DEAUTH_STATION;
	wrq.u.data.pointer = (caddr_t)&buff;
        wrq.u.data.length = sizeof(buff);
	if ( ioctl(skfd, IEEE80211_IOCTL_GETADDRPARAM,&wrq) >= 0)
	{
		sock = (struct sockaddr *) buff;
		DEBUG_MSG(nsc.deAuthenticationStation, "%02X:%02X:%02X:%02X:%02X:%02X",
			(u_char)sock->sa_data[0],(u_char)sock->sa_data[1],(u_char)sock->sa_data[2],(u_char)sock->sa_data[3],(u_char)sock->sa_data[4],(u_char)sock->sa_data[5]);
		for(i = 0; i < 6; i++)
			nsc.deAuthenticationStation[i] = (u_char)sock->sa_data[i];
	}
	DEBUG_MSG("nsc.deAuthenticationStation %s\n",nsc.deAuthenticationStation);
			
	/*Authentication Fail Status*/
	wrq.u.data.flags = IEEE80211_PARAM_AUTH_FAILSTATUS;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
		nsc.authenticateFailStatus = wrq.u.param.value;
	DEBUG_MSG("\nnsc.authenticateFailStatus %u\n",nsc.authenticateFailStatus);

	/*Authentication Fail Station */
	memset(nsc.authenticateFailStation, 0, (MACADDR_LEN + 1));
	wrq.u.data.flags = IEEE80211_PARAM_AUTH_FAILSTATION;
	wrq.u.data.pointer = (caddr_t)&buff;
        wrq.u.data.length = sizeof(buff);
	if ( ioctl(skfd, IEEE80211_IOCTL_GETADDRPARAM,&wrq) >= 0)	
	{
		sock = (struct sockaddr *) buff;
		DEBUG_MSG(nsc.authenticateFailStation, "%02X:%02X:%02X:%02X:%02X:%02X",
			(u_char)sock->sa_data[0],(u_char)sock->sa_data[1],(u_char)sock->sa_data[2],(u_char)sock->sa_data[3],(u_char)sock->sa_data[4],(u_char)sock->sa_data[5]);
		for(i = 0; i < 6; i++)
			nsc.authenticateFailStation[i] = (u_char)sock->sa_data[i];
	}
	DEBUG_MSG("\nnsc.authenticateFailStation %s\n",nsc.authenticateFailStation);
	
	/*Multi Domain Capability Implemented*/
	wrq.u.param.value = IEEE80211_PARAM_MULDOM_CAP_IM;
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.multidomaincapability_impl = wrq.u.param.value;*/
		if(wrq.u.param.value)
			 nsc.multidomaincapability_impl = 1;
		else
			 nsc.multidomaincapability_impl = 1;
	}	
	DEBUG_MSG("\nnsc.multidomaincapability_impl %u\n",nsc.multidomaincapability_impl);
	
	/*Multi Domain Capability Enabled*/
	wrq.u.param.value = IEEE80211_PARAM_MULDOM_CAP_EN;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.multidomaincapability_ena = wrq.u.param.value;*/
		if(wrq.u.param.value)
			nsc.multidomaincapability_ena = 1;
		else
			nsc.multidomaincapability_ena = 2;
	}
	DEBUG_MSG("nsc.multidomaincapability_ena %u\n",nsc.multidomaincapability_ena);

	/*Country String TODO*/
	wrq.u.data.flags = IEEE80211_PARAM_COUNTRYSTRING;
	wrq.u.data.pointer = country;
	wrq.u.data.length = strlen(country);	
	if( ioctl(skfd, IEEE80211_IOCTL_GETSTRPARAM,&wrq) >= 0) {
		strncpy(nsc.countrystring,country,4);
		nsc.countrystring[4] = '\0';
	}
	DEBUG_MSG("sc.countrystring %s\n",nsc.countrystring);

	
	/*Spectrum Management Implemented*/	
	wrq.u.param.value = IEEE80211_PARAM_SPECMGMT_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.spectrummanagement_impl = wrq.u.param.value;*/
		if(wrq.u.param.value)
			nsc.spectrummanagement_impl = 1;
		else
			nsc.spectrummanagement_impl = 2;
	}
	DEBUG_MSG("nsc.spectrummanagement_impl %u\n",nsc.spectrummanagement_impl);

	/*Spectrum Management Required*/
	wrq.u.param.value = IEEE80211_PARAM_SPECMGMT_REQ;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		if(wrq.u.param.value)
			nsc.spectrummanagement_req = 1;
		else
			nsc.spectrummanagement_req = 2;
	}
	DEBUG_MSG("nsc.spectrummanagement_req %u\n",nsc.spectrummanagement_req);
	
	/*RSNAOption IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_RSNA_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.RSNAOption_impl = wrq.u.param.value;*/
		if(wrq.u.param.value)
			nsc.RSNAOption_impl = 1;
		else
			nsc.RSNAOption_impl = 2;
	}
	DEBUG_MSG("nsc.RSNAOption_impl %u\n",nsc.RSNAOption_impl);

	/*RSNA Pre Auth Option IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_RSNA_PREAUTH_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		nsc.RSNAPreAuth_impl = wrq.u.param.value;
                if(wrq.u.param.value)
			nsc.RSNAPreAuth_impl = 1;
		else
			nsc.RSNAPreAuth_impl = 2;
	}
	DEBUG_MSG("nsc.RSNAPreAuth_impl %u\n",nsc.RSNAPreAuth_impl);

	/*Regulatory IMPL*/
	wrq.u.param.value = IEEE80211_PARAM_REGCLASS_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.regulatory_impl = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.regulatory_impl = 1;
		else
			nsc.regulatory_impl = 2;
	}
	DEBUG_MSG("nsc.regulatory_impl %u\n",nsc.regulatory_impl);

	/*Regulatory Req*/
	wrq.u.param.value = IEEE80211_PARAM_REGCLASS_REQ;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.regulatory_req = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.regulatory_req = 1;
		else
			nsc.regulatory_req = 2;
	}
	DEBUG_MSG("nsc.regulatory_req %u\n",nsc.regulatory_req);

	/*QOS Option Ipmlemented*/
	wrq.u.param.value = IEEE80211_PARAM_QOS_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.QosOption_impl = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.QosOption_impl = 1;
		else
			nsc.QosOption_impl = 2;
	}
	DEBUG_MSG(" nsc.QosOption_impl %u\n",nsc.QosOption_impl);

	/*immediate_Ack*/
	wrq.u.param.value = IEEE80211_PARAM_IMM_BLOCKACK_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.immediate_Ack = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.immediate_Ack = 1;
		else
			nsc.immediate_Ack = 2;
	}
	DEBUG_MSG("nsc.immediate_Ack %u\n",nsc.immediate_Ack);

	/*delayed_Ack */
	wrq.u.param.value = IEEE80211_PARAM_DELAY_BLOCKACK_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.delayed_Ack = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.delayed_Ack = 1;
		else
			nsc.delayed_Ack = 2;
	}		
	DEBUG_MSG("nsc.delayed_Ack %u\n",nsc.delayed_Ack);

	/*directedOption */
	wrq.u.param.value = IEEE80211_PARAM_DIRECT_OPT_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.directedOption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.directedOption = 1;
		else
			nsc.directedOption = 2;
	}
	DEBUG_MSG("nsc.directedOption %u\n",nsc.directedOption);


	/*APSDOption*/
	wrq.u.param.value = IEEE80211_PARAM_APSD_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.APSDOption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.APSDOption = 1;
		else
			nsc.APSDOption = 2;
	}
	DEBUG_MSG("nsc.APSDOption %u\n",nsc.APSDOption);

	/*QAckOption*/
	wrq.u.param.value = IEEE80211_PARAM_QACK_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.QAckOption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.QAckOption = 1;
		else
			nsc.QAckOption = 2;
	}
	DEBUG_MSG("nsc.QAckOption %u\n",nsc.QAckOption);

	/*QBSSLoadOption*/
	wrq.u.param.value = IEEE80211_PARAM_QBSSLOAD_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.QBSSLoadOption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.QBSSLoadOption = 1;
		else
			nsc.QBSSLoadOption = 2;
	}
	DEBUG_MSG("nsc.QBSSLoadOption %u\n",nsc.QBSSLoadOption);

	/*QueueReqOption*/
	wrq.u.param.value = IEEE80211_PARAM_QUEUEREQ_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
		/*nsc.QueueReqOption = wrq.u.param.value;*/
                if(wrq.u.param.value)
			nsc.QueueReqOption = 1;
		else	
			nsc.QueueReqOption = 2;	
	}
	DEBUG_MSG("nsc.QueueReqOption %u\n",nsc.QueueReqOption);

	/*TXOPReqOption*/
	wrq.u.param.value = IEEE80211_PARAM_TXOPREQ_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.TXOPReqOption = 1;
		else
			nsc.TXOPReqOption = 2;
	}
	DEBUG_MSG("nsc.TXOPReqOption %u\n",nsc.TXOPReqOption);

	/*MoreAckOption*/
	wrq.u.param.value = IEEE80211_PARAM_MOREDATA_ACK_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.MoreAckOption = 1;
		else
			nsc.MoreAckOption = 2;
	}
	DEBUG_MSG("nsc.MoreAckOption %u\n",nsc.MoreAckOption);

	/*AssociateNQBSS*/
	wrq.u.param.value = IEEE80211_PARAM_ASSOCIN_NQBSS;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.AssociateNQBSS = 1;
		else
			nsc.AssociateNQBSS = 2;
	}
	DEBUG_MSG("nsc.AssociateNQBSS %u\n",nsc.AssociateNQBSS);

	/*DLSAllowedQBSS*/
	wrq.u.param.value = IEEE80211_PARAM_ALLOWIN_QBSS;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.DLSAllowedQBSS = 1;
		else
			nsc.DLSAllowedQBSS = 2;
	}
	DEBUG_MSG("DLSAllowedQBSS %u\n",nsc.DLSAllowedQBSS);

	/*DLSAllowed*/
	wrq.u.param.value = IEEE80211_PARAM_DLSALLOWED;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.DLSAllowed = 1;
		else
			nsc.DLSAllowed = 2;
	}
	DEBUG_MSG("DLSAllowed %u\n",nsc.DLSAllowed);

	/*HighThroughputOption*/
	wrq.u.param.value = IEEE80211_PARAM_HIGH_TPT_IM;	
	if( ioctl(skfd, IEEE80211_IOCTL_GETPARAM,&wrq) >= 0)
	{
                if(wrq.u.param.value)
			nsc.HighThroughputOption = 1;
		else
			nsc.HighThroughputOption = 2;
	}
	DEBUG_MSG("HighThroughputOption %u\n",nsc.HighThroughputOption);
	

	/*Add List*/
	addList(&nsc_list,(char *)&nsc,sizeof(nsc));		
}

int load_Table (void (*load_list)(int,char *),list_T *list_ptr)
{
	int skfd;
	char ifname[256],bfr[1024],vapname[256];
	FILE *fp;
	struct iwreq wrq;
	char *s,*t;
	int i;
        struct ifreq ifr;
	flushlist(list_ptr);
	list_int(list_ptr);

	skfd = socket(AF_INET,SOCK_DGRAM, 0);
	if(skfd < 0)
	{
		//snmp_log(LOG_ERR,"IEEE802 query socket open failed\n");
		DEBUG_MSG("\nIEEE802 query socket open failed\n");
		return ERR;
	}

	fp = fopen(PROC_NET_DEV,"r");
	if(fp)
	{
              snmp_log(LOG_ERR,"opened the file \n");
		while( fgets(bfr, sizeof(bfr), fp) )
		{
			if( strstr(bfr,":") )
			{
				s = bfr;
				t = ifname;
				while( isspace( *s))
					s++;
				while( *s != ':')
					*t++ = *s++;
				*t = '\0';
				/*Verify whether it is a wireless interface*/
				strncpy(wrq.ifr_name,ifname,IFNAMSIZ);

                             if(!strncmp(ifname,"wifi0vap",8)){    
				for(i=0; i<=7; i++) {
                                     sprintf(vapname, "%s%d","wifi0vap",i);
                                       if(!strcmp(vapname,ifname)) {
  					     snmp_log(LOG_ERR,"ifName is \n %s",ifname);                                    
                		             memset(&ifr, 0, sizeof(ifr));
                		             strncpy(ifr.ifr_name, ifname , strlen(ifname));
                		             ioctl(skfd, SIOCGIFFLAGS, &ifr);
                                                  if((ifr.ifr_flags & IFF_UP)) { 
                                                         snmp_log(LOG_ERR,"ifName \n %s",ifr.ifr_name);
                                                         (void) strncpy(wrq.ifr_name, ifname ,strlen(ifname));
			   	                         load_list(skfd, ifname);
                                                  }
		  	               }	
                                }
			    }	
		       }
	       }
     }
    fclose (fp);
    close (skfd);
    return 0;
}

